/****************************************************************************/
#include <iostream>
#include <stdio.h>
#include<string.h>
#include <float.h>
#include <assert.h>
#include "BranchingStructure.h"
#include "Graph.h"
#include "const.h"
#include "xgraphic.h"
#include "xmenus.h"
#include "xstats.h"
#include "xdrw.h"
#include "xrout.h"
/***************************************************************************/
using namespace std;
/***************************************************************************/
BRNODE *BranchingStructure::newNode(void)
{
	memory_used+=sizeof(BRNODE);
	return(new BRNODE);
}
/***************************************************************************/
BRNODE *BranchingStructure::initNode(void) 
{
	BRNODE *node;
	node=newNode();
	node->up=NULL;
	node->prefixPath=NULL;
	node->node=(-1);
	node->order=(-1);
	node->num=num;
	num++;
	return(node);
}
/***************************************************************************/
BranchingStructure::BranchingStructure(void) 
{
	memory_used+=sizeof(BranchingStructure);
	root=NULL;
	order=0;
	num=0;
}

BranchingStructure::BranchingStructure(BranchingStructure& p)
{
   root=new BRNODE[sizeof(p.root)];
   memcpy(root,p.root,sizeof(p.root)*sizeof(BRNODE));
   order=p.order;
   num=p.num;
}
/***************************************************************************/
BranchingStructure::~BranchingStructure(void)
{
	if (root!=NULL) destroyNode(root);
}
/******************************************************************************/
void BranchingStructure::destroyNode(BRNODE *node)
{
	BRNODE *child;
	BRLIST::iterator it;

	// Delete all the children
	while(FOREVER) {
		if (node->children.empty()) break;
		it=node->children.begin();
		child=(*it);
		destroyNode(child);
		node->children.erase(it);
	}
	delete node->prefixPath;
	delete node;
}
/******************************************************************************/
BRNODE *BranchingStructure::createRootNode(int node)
{
	root=initNode();
	root->up=NULL;
	root->node=node;
	root->prefixPath=new Path;
	return(root);
}
/******************************************************************************/
// Add a new child from start that ends in a new node end
BRNODE *BranchingStructure::addLeaf(Graph *G, BRNODE *u, Path *p)
{
	int i,start;
	BRNODE *tp;
	Edge edge;

	// If debug show what we are adding
	if (Sys.getDebug()) {
		Sys.logString("Adding leaf for u\n");
		G->logPath(*p);
	}

	tp=initNode();		    // Create memory for node and inits data
	tp->up=u;				// Set parent

	// Create the new prefixPath for tp
	start=(-1);
	for (i=0;i<p->nList;i++) {
		edge=G->getEdge(p->list[i]);
		if (edge.p1==u->node) {
			// Store only the prefix path
			tp->prefixPath=new Path;
			tp->prefixPath->nList=p->nList-i;
			tp->prefixPath->list=new int[tp->prefixPath->nList];
			tp->prefixPath->cost=0.0;
			start=i;
		}
		// Store the path
		if (start>=0) {
			tp->prefixPath->list[i-start]=p->list[i];
			tp->prefixPath->cost+=edge.cost;
		}
	}

	// ??? Debug 
	if (start==(-1)) {
		put_error("Error:u not found on path");
		if (Sys.getLogging()) {
			Sys.logString("Error:u not found on ");
			G->logPath(*p);
			walkTree(G);
			Sys.stopLogging();
		}
		return(NULL);
	}

	// Last edge destination node is tp->node
	tp->node=edge.p2;

	// Add tp as a child of u
	u->children.push_back(tp);

	// Set the order of this path
	tp->order=order;
	order++;

	// If debug show how structure has changed
	if (Sys.getDebug()) walkTree(G);
	return(tp);
}
/******************************************************************************/
// This breaks a branch and inserts a new node
BRNODE *BranchingStructure::addBranch(Graph *G, BRNODE *u, BRNODE *v, Path *p)
{
	int i,j,found,start,offset=0;
	BRNODE *w;
	Path *newpath;
	BRLIST::iterator it;
	Edge edge;	
	
	// If debug show what we are adding
	if (Sys.getDebug() && Sys.getLogging()) {
		Sys.logString("Adding branch for u ");
		G->logPath(*p);
	}

	// Create and init a new node
	w=initNode();
	w->up=u;

	// Find the key
	// Note:We should order the children so we can find one quickly
	found=FALSE;
	for (it=u->children.begin();it!=u->children.end();++it) {
		if ((*it)==v) {
			u->children.erase(it);
			found=TRUE;
			break;
		}
	}

	// Remove v from his parent
	if (found==FALSE) {
		put_error("Error:addBranch:v not found as child of u");
		if (Sys.getLogging()) {
			Sys.logString("Error:addBranch:v not found as child of u\n");
			walkTree(G);
			Sys.stopLogging();
		}
		return(NULL);
	}

	// Add w as a child of u
	u->children.push_back(w);

	// Add v as a child of w the new node
	w->children.push_back(v);
	v->up=w;

	// Find node for w - last point in common between path p and p(u,v)
	start=(-1);
	for (i=0;i<p->nList;i++) {
		edge=G->getEdge(p->list[i]);

		if (edge.p1==u->node) start=i;				// Start position in path = u node
		else if (start>=0) {
			offset=i-start;

			// If we run out of prefix path or nodes are different
			if (offset>=v->prefixPath->nList) {
				put_error("Error:Cannot find a node for w");
				break;
			}

			// w node is the last node in common with v->prefpath
			// so it is the first edge that is different
			if (p->list[i]!=v->prefixPath->list[offset]) {	
				w->node=edge.p1;	
				break;
			}
		}
	}

	// Debug errors
	if (start==(-1)) {
		put_error("Error:Cannot find u on path");
		if (Sys.getLogging()) {
			Sys.logString("Error:Cannot find u on path\n");
			walkTree(G);
			Sys.stopLogging();
		}
		return(NULL);
	}

	// Debug errors
	if (w->node<0) {
		put_error("Error:Cannot find split node w on path");
		if (Sys.getLogging()) {
			Sys.logString("Error:Cannot find split node w on path\n");
			walkTree(G);
			Sys.stopLogging();
		}
		return(NULL);
	}

	assert(offset>0);

	// Find the prefixPath of w
	w->prefixPath=new Path;
	w->prefixPath->nList=offset;
	w->prefixPath->list=new int[w->prefixPath->nList];
	w->prefixPath->cost=0.0;

	// Divide v's prefixPath into w and v's new prefixPath
	for (j=0;j<w->prefixPath->nList;j++) {
		w->prefixPath->list[j]=v->prefixPath->list[j];

		// Calculate the cost of this edge
		edge=G->getEdge(w->prefixPath->list[j]);
		w->prefixPath->cost+=edge.cost;
	}

	// Find prefixPath of v
	newpath=new Path;
	newpath->nList=v->prefixPath->nList-w->prefixPath->nList;
	newpath->list=new int[newpath->nList];
	newpath->cost=0.0;
	for (j=w->prefixPath->nList;j<v->prefixPath->nList;j++) { 
		newpath->list[j-w->prefixPath->nList]=v->prefixPath->list[j];
		edge=G->getEdge(v->prefixPath->list[j]);
		newpath->cost+=edge.cost;
	}

	delete v->prefixPath;
	v->prefixPath=newpath;

	// If debug show how structure has changed
	if (Sys.getDebug()) walkTree(G);

	return(w);
}
/***************************************************************************/
// This breaks a branch and inserts a new node
BRNODE *BranchingStructure::addNewBranch(Graph *G, BRNODE *u, BRNODE *v, Path *p)
{
	int i,j,k,e,e1,e2,foundW=FALSE;
	BRNODE *w,*t,*child,*parent,*firstChild;
	BRLIST::iterator it;
	Edge edge;	
	int count,total;
	BRNODE *node;
	
	// If debug show what we are adding
	if (Sys.getDebug() && Sys.getLogging()) {
		Sys.logString("Adding branch for u ");
		G->logPath(*p);
	}

	// We need the path from u to v but we only have from v to u so compute
	// u to v and store in a list
	// First count the number in the path
	count=0;
	node=v;
	while(FOREVER) {
		if (node==NULL) break;
		count++;
		node=node->up;
	}
	PBRNODE *brlist=new PBRNODE[count];
	// Now store
	total=count;
	count=0;
	node=v;
	while(FOREVER) {
		if (node==NULL) break;
		brlist[total-count-1]=node;
		count++;
		node=node->up;
	}

	// Find w's node
	for (j=0,k=0;j<count-1;j++) {
		parent=brlist[j];
		child=brlist[j+1];

		// Compare edges until they separate
		for (i=0;i<child->prefixPath->nList;i++,k++) {
			e1=child->prefixPath->list[i];
			e2=p->list[k];
			if (e1!=e2) {

				// Compute remainder path which is added to leaf of w or other node
				Path *newPath=new Path;
				newPath->nList=p->nList-k;
				newPath->list=new int[newPath->nList];
				newPath->cost=0.0;
				for (j=k;j<p->nList;j++) {
					newPath->list[j-k]=p->list[j];
					edge=G->getEdge(p->list[j]);
					newPath->cost+=edge.cost;
				}

				// If w is already a node in the structure add leaf	to that node
				if (i==0) {
					delete [] brlist;
					return(addLeaf(G,parent,newPath));
				}

				w=initNode();
				edge=G->getEdge(e1);
				w->node=edge.p1;
				t=addLeaf(G,w,newPath);


				// Add w as the first child of the parent
				// If this child was the first child
				firstChild=(*(parent->children.begin()));
				if (child==firstChild) {
					// Delete child from parent
					parent->children.erase(parent->children.begin());
					// Add w as first child
					parent->children.push_front(w);
				}
				else {
					// Find child
					for (it=parent->children.begin();it!=parent->children.end();it++) {
						if ((*it)==child) {
							//parent->children.erase(it);
							(*it)=w;
							break;
						}
					}
					//parent->children.push_back(w);
				}
				w->up=parent;

				// Add child to w (This will always be first child)
				w->children.push_front(child);
				child->up=w;

				foundW=TRUE;
				break;
			}
		}
		if (foundW==TRUE) break;
	}

	// Split the edges into w's prefix path and child's prefix path
	// Count the number of edges in w
	for (i=0;i<child->prefixPath->nList;i++) {
		edge=G->getEdge(child->prefixPath->list[i]);
		if (edge.p1==w->node) break;
	}

	// i is now the number of edges that should be in w's prefixpath
	// Find the prefixPath of w
	w->prefixPath=new Path;
	w->prefixPath->nList=i;
	w->prefixPath->list=new int[w->prefixPath->nList];
	w->prefixPath->cost=0.0;

	// Find prefixPath of child
	Path *newPath=new Path;
	newPath->nList=child->prefixPath->nList-w->prefixPath->nList;
	newPath->list=new int[newPath->nList];
	newPath->cost=0.0;

	// Divide v's prefixPath into w and v's new prefixPath
	for (j=0;j<child->prefixPath->nList;j++) {
		e=child->prefixPath->list[j];
		edge=G->getEdge(e);
		if (j<i) {
			w->prefixPath->list[j]=e;
			w->prefixPath->cost+=edge.cost;
		}
		else {
			newPath->list[j-i]=e;
			newPath->cost+=edge.cost;
		} 
	}

	// Set child's new prefixPath
	delete child->prefixPath;
	child->prefixPath=newPath;

	// If debug show how structure has changed
	if (Sys.getDebug()) walkTree(G);

	delete [] brlist;

	return(t);
}
/***************************************************************************/
// If we call this we are logging
void BranchingStructure::walkTree(Graph *G)
{
	Sys.logString("Branching structure -------------------------------\n");
	walkNode(G,root);
	Sys.logString("---------------------------------------------------\n");
}
/***************************************************************************/
void BranchingStructure::walkNode(Graph *G, BRNODE *node)
{
	BRNODE *child;
	BRLIST::iterator it;
	char msg[128];

	if (node->prefixPath!=NULL) {
		if ((node->prefixPath->nList)>0)
			sprintf(msg,"BR node:%1d %p parent:%p lead:%d prefix",
				node->node,node,node->up,node->prefixPath->list[0]);
		else 
			sprintf(msg,"BR node:%1d %p parent:%p lead:None prefix",
				node->node,node,node->up);
		Sys.logString(msg);
		G->logPath(*(node->prefixPath));
	}

	for (it=node->children.begin();it!= node->children.end();++it) {
		child=(*it);
		walkNode(G,child);
	}
	return;
}
/***************************************************************************/
// Note:Calling routine is responsible for deleting path
Path *BranchingStructure::computeTotalPrefix(BRNODE *node0)
{
	int i;
	int n;
	BRNODE *node;
	static Path p;

	// Count the number of edges in the total prefix path
	n=0;
	node=node0;
	while(FOREVER) {
		if (node==NULL) break;
		n+=node->prefixPath->nList;
		node=node->up;
	}

	if (n>p.nList) {
		delete p.list;
		p.list=new int[n];
	}
	p.nList=n;
	p.cost=0.0;

	n=0;
	node=node0;
	while(FOREVER) {
		if (node==NULL) break;
		for (i=0;i<node->prefixPath->nList;i++) 
			p.list[p.nList-node->prefixPath->nList-n+i]=node->prefixPath->list[i];
		n+=node->prefixPath->nList;
		p.cost+=node->prefixPath->cost;
		node=node->up;
	}

	return(&p);	
}
/***************************************************************************/
void BranchingStructure::computePaths(Graph *G, Path *paths, int *npath)
{
	Path prefix(G->getNumNodes());
	(*npath)=0;
	computePaths(G,&prefix,0,0.0,root,paths,npath);
}
/***************************************************************************/
void BranchingStructure::computePaths(Graph *G,	// Pointer to graph
		Path *prefix,							// Prefix path so far
		int size,								// Accumulated size of prefix
		double cost,							// Accumulated cost of prefix
		BRNODE *node,							// Tranverse node
		Path *paths,							// List of paths
		int *npath)								// Number of paths so far
{
	int i;
	BRNODE *child;
	BRLIST::iterator it;
	Edge edge;

	// If no children then this is a path
	if (node->children.empty()) {
		int n=node->order;
		//n=(*npath);	// Shorthand

		// Init this path
		paths[n].nList=node->prefixPath->nList+size;
		paths[n].list=new int[node->prefixPath->nList+size];
		paths[n].cost=cost;

		for (i=0;i<size;i++) {
			paths[n].list[i]=prefix->list[i];
		}
		for (i=0;i<node->prefixPath->nList;i++) {
			paths[n].list[size+i]=node->prefixPath->list[i];
			edge=G->getEdge(node->prefixPath->list[i]);
			paths[n].cost+=edge.cost;
		}

		(*npath)++;
	}
	else {
		// This is essentially an error condition
		// Put in fix so we can debug ?????
		if (node->prefixPath==NULL) return;

		// Add our prefix points to the prefix
		for (i=0;i<node->prefixPath->nList;i++) { 
			prefix->list[i+size]=node->prefixPath->list[i];
			edge=G->getEdge(node->prefixPath->list[i]);
			cost+=edge.cost;	
		}
		size+=node->prefixPath->nList;
		//cost+=node->prefixPath->cost;

		// Go through the children
		for (it=node->children.begin();it!=node->children.end();++it) {
			child=(*it);
			computePaths(G,prefix,size,cost,child,paths,npath);
		}
	}
	return;
}
/***************************************************************************/
