/******************************************************************************/
/*
  BranchingStructure.h
  Created Oct, 2001
*/
/******************************************************************************/
#ifndef BranchingStructure_h
#define BranchingStructure_h
/******************************************************************************/
#include "BranchingStructureNode.h"
#include "Graph.h"
#include "Path.h"
/******************************************************************************/
// This is the structure for storing paths information in the heap
struct pathInfo {
	Path *path;
	BRNODE *u; // Equivalence Type C(u,v);
	BRNODE *v; // For class type C(u) brnode2=NULL
};
typedef pathInfo PATH_INFO;
/******************************************************************************/
class BranchingStructure {
	BRNODE *root;			// Pointer to the root
	int order;				// Order in which the paths are created
	int num;				// Number of node created
	void destroyNode(BRNODE *node);
	BRNODE *newNode(void);
	BRNODE *initNode(void);
public:
	BranchingStructure(void);
        BranchingStructure(BranchingStructure& P);
	~BranchingStructure(void);
	BRNODE *createRootNode(int node);


	BRNODE *addLeaf(Graph *G, BRNODE *u, Path *p);
	BRNODE *addBranch(Graph *G, BRNODE *start, BRNODE *end, Path *p);
	void walkTree(Graph *G);
	void walkNode(Graph *G, BRNODE *node);
	Path *computeTotalPrefix(BRNODE *node0);
	BRNODE *addNewBranch(Graph *G, BRNODE *u, BRNODE *v, Path *p);

	// Compute all paths from the structure
	void computePaths(Graph *G, Path *paths, int *npath);
	void computePaths(Graph *G,						// Pointer to graph
		Path *prefix,								// Prefix path so far
		int size,									// Accumulated number of edges
		double cost,								// Accumulated cost
		BRNODE *node,								// Tranverse node
		Path *paths,								// List of paths
		int *npath);								// Number of paths so far
};
/***************************************************************************/
#endif
/***************************************************************************/
