/***************************************************************************/
/*
	FibonacciHeap.h
	Written by Matthew Maxel
	Created Dec 10, 2001
*/
/******************************************************************************/
#ifndef FibonacciHeap_h
#define FibonacciHeap_h 1
/******************************************************************************/
struct FibonacciHeap_node {
	struct FibonacciHeap_node *up;
	struct FibonacciHeap_node *child;
	void *item;
	double key;
	int rank;												// Number of children
	unsigned int marked:1;
	unsigned int is_root:1;
	struct FibonacciHeap_node *next;								// Next node in circular chain
	struct FibonacciHeap_node *prev;								// Prev node in circular chain
};

typedef FibonacciHeap_node FNODE;
typedef FNODE * PFNODE;

class FibonacciHeap {
	// Private Data
	FNODE *min;						// Pointer the minimum root
	int max_rank;					// Maximum rank of any node
	int nroot;						// Number of roots
	int nnode;						// Number of nodes

	int myChunk;					// My chunk of memory

	// Private Functions
	void meld(FNODE *node);
	void destroy_node(FNODE *node);					// Deletes node and all descendants
	int compare_nodes(FNODE *node1, FNODE *node2);
	void add_child(FNODE *parent, FNODE *node);
	FNODE *create_node(void);
	FNODE *link(FNODE *node1, FNODE *node2);
	void add_root(FNODE *node);
	int check_node(FNODE *node);
	void check_heap(void);
	void destroyData(FNODE *node);
public:
	FibonacciHeap(void);
	~FibonacciHeap(void);
	void *delete_node(FNODE *node);						// Delete one node
	FNODE *insert_item(void *item, double key);
	void *get_min(void);
	void *delete_min(void);
	void decrease_key(FNODE *node, double new_key);
	void destroyData(void);
	void clear(void);
};
/***************************************************************************/
#endif
/***************************************************************************/
