/******************************************************************************/
/*
	Graph.cpp
	Created Oct, 2001
*/
/******************************************************************************/
#include <math.h>
#include <fstream>
#include <string.h>
#include <sstream>
#include <assert.h>
#include <cstdlib>
#include <map>
/******************************************************************************/
#include "Graph.h"
#include "BranchingStructure.h"
#include "FibonacciHeap.h"
#include "NodeLabels.h"
#include "Path.h"
#include "LinkList.h"
#include "DataParam.h"
#include "Node.h"
/******************************************************************************/
#include "const.h"
#include "xstats.h"
#include "xmenus.h"
#include "xutil.h"
#include "xdrw.h"
#include "xgraphic.h"
#include "xprim.h"
#include "xcompare.h"
#include "colors.h"
#include "ret.h"
#include "xrout.h"
/******************************************************************************/
// Log this path to a file
void Graph::logPath(const Path& path)
{
	char msg[1024];
	formatPath(path,msg);
	Sys.logString(msg);
	Sys.logString("\n");
}
/***************************************************************************/
// Format the path with the name
void Graph::formatPath(char *pathName, const Path& path, char *totalMsg)
{
	char pathMsg[1024];
	if (pathName[0]) {
		formatPath(path,pathMsg);
		sprintf(totalMsg,"%s %s",pathName,pathMsg);
                //sprintf(totalMsg,"%s",pathName);
	}
	else formatPath(path,totalMsg);
}
/***************************************************************************/
Graph::Graph(void)
{
	p0.npts=0;
	nodes=NULL;
	p0.nedges=0;
	edges=NULL;
	startNode=0;
	endNode=0;
	labels=NULL;
	hybridNumber=5;

	scratchLabels=NULL;		// Used for subgraphs
	scratchLabels2=NULL;	// Used for naive shortest paths
	scratchEdges=NULL;		// Duplicate edges
	shortestPath=NULLPATH;	// Init shortest path info
	
	forwardIndex=NULL;		// Index to nodes sorted by source node
	forwardEdgeList=NULL;	// Used to find a list of edges from a source node
	reverseIndex=NULL;		// Index to nodes sorted by destination node
	reverseEdgeList=NULL;	// Used to find a list of edges to a destination node

	reverseEdges=NULL;
	edgeKey=NULL;

	numBadCases=0;
	numKSP=0;
	totalKSPLengths=0;
	numBranchPaths=0;
	totalBranchLengths=0;
	numDijkstras=0;
	numNodesScanned=0;

	ordered=FALSE;
}
Graph::Graph(Graph &p)
{
   startNode=p.startNode;
   endNode=p.endNode;
   numBadCases=p.numBadCases;
   numBranchPaths=p.numBranchPaths;
   totalBranchLengths=p.totalBranchLengths;
   numKSP=p.numKSP;
   totalKSPLengths=p.totalKSPLengths;
   numDijkstras=p.numDijkstras;
   numNodesScanned=p.numNodesScanned;
   ordered=p.ordered;
   labelsValid=p.labelsValid;
   nodes=new Node[sizeof(p.nodes)];
   memcpy(nodes,p.nodes,sizeof(p.nodes)*sizeof(Node));
   edges=new Edge[sizeof(p.edges)];
   memcpy(edges,p.edges,sizeof(p.edges)*sizeof(Edge));
   forwardIndex=new int[sizeof(p.forwardIndex)];
   memcpy(forwardIndex,p.forwardIndex,sizeof(p.forwardIndex)*sizeof(int));
   //EdgeList *forwardEdgeList;
   forwardEdgeList=new EdgeList[sizeof(p.forwardEdgeList)];
   memcpy(forwardEdgeList,p.forwardEdgeList,sizeof(p.forwardEdgeList)*sizeof(EdgeList));
  //int *reverseIndex;
  reverseIndex=new int[sizeof(p.reverseIndex)];
   memcpy(reverseIndex,p.reverseIndex,sizeof(p.reverseIndex)*sizeof(int));
  //EdgeList *reverseEdgeList;
  reverseEdgeList=new EdgeList[sizeof(p.reverseEdgeList)];
   memcpy(reverseEdgeList,p.reverseEdgeList,sizeof(p.reverseEdgeList)*sizeof(EdgeList));
//NodeLabels *labels;
  labels=new NodeLabels[sizeof(p.labels)];
   memcpy(labels,p.labels,sizeof(p.labels)*sizeof(NodeLabels));
//NodeLabels *scratchLabels;
  scratchLabels=new NodeLabels[sizeof(p.reverseIndex)];
   memcpy(scratchLabels,p.scratchLabels,sizeof(p.scratchLabels)*sizeof(NodeLabels));
//NodeLabels *scratchLabels2;
 scratchLabels2=new NodeLabels[sizeof(p.scratchLabels2)];
   memcpy(scratchLabels2,p.scratchLabels2,sizeof(p.scratchLabels2)*sizeof(NodeLabels));
//Edge *scratchEdges;
   scratchEdges=new Edge[sizeof(p.scratchEdges)];
   memcpy(scratchEdges,p.scratchEdges,sizeof(p.scratchEdges)*sizeof(Edge));
//Edge *reverseEdges;
 reverseEdges=new Edge[sizeof(p.reverseEdges)];
   memcpy(reverseEdges,p.reverseEdges,sizeof(p.reverseEdges)*sizeof(Edge));
//EdgeKey *edgeKey;
 edgeKey=new EdgeKey[sizeof(p.edgeKey)];
   memcpy(edgeKey,p.edgeKey,sizeof(p.edgeKey)*sizeof(EdgeKey));
}

/******************************************************************************/
// Free all the items 
Graph::~Graph(void)
{
	freeAll();
}
/******************************************************************************/
// Compute a list for and node of all edges eminating(edges) and all edges
// terminating(reverse_edgelist)
void Graph::computeEdgeLists(void)
{
	int i,c,n,e,sorted=TRUE,showInfo;

	showInfo=Sys.getShowInfo();

	// Sort indexes
	current_graph=this;	// Needs to be global for the quicksort

	if (showInfo) 
		//show_msg("Sorting edges");

	// Make sure these are free
	if (forwardIndex!=NULL) {
		delete forwardIndex;
		forwardIndex=NULL;
	}
	if (reverseIndex!=NULL) {
		delete reverseIndex;
		reverseIndex=NULL;
	}

	// Initialize forward and backwards index
	forwardIndex=new int[p0.nedges];
	reverseIndex=new int[p0.nedges];
	for (i=0;i<p0.nedges;i++) {
		forwardIndex[i]=i;
		reverseIndex[i]=i;
	}

	// Sort the forwardIndex to the edges based on source node
	if (showInfo) 
		//show_msg("Sorting edges based on source node with quick sort");
	qsort(forwardIndex,p0.nedges,sizeof(int),fastCompare);

	// Sort the reverseIndex to the edges based on the destination node
	if (showInfo) 
		//show_msg("Sorting edges based on destination node with quick sort");
	qsort(reverseIndex,p0.nedges,sizeof(int),fastRCompare);

	// Create a list of edges for a source node
	if (showInfo) 
		//show_msg("Computing forward index");
	forwardEdgeList=new EdgeList[p0.npts];

	// Init forward edge list
	for (i=0;i<p0.npts;i++) {
		forwardEdgeList[i].start=0;
		forwardEdgeList[i].n=0;
	}

	// Compute a start of EdgeList and number of edges for each node
	c=0;	// Number of edges of this type
	n=0;	// Current start node
	forwardEdgeList[0].start=0;
	for (i=0;i<p0.nedges;i++) {
		e=forwardIndex[i];

		if (edges[e].p1!=n) {
			forwardEdgeList[n].n=c;
			n=edges[e].p1;
			forwardEdgeList[n].start=i;
			c=0;
		}
		c++;
	}
	forwardEdgeList[n].n=c;

	// Create a list of edges for a destination node
	if (showInfo) 
		//show_msg("Computing reverse index");
	reverseEdgeList=new EdgeList[p0.npts];

	// Init reverse list
	for (i=0;i<p0.npts;i++) {
		reverseEdgeList[i].start=0;
		reverseEdgeList[i].n=0;
	}

	c=0;	// Number of edges of this type
	n=0;	// Current start node
	n=reverseEdges[reverseIndex[0]].p1;
	reverseEdgeList[n].start=0;
	for (i=0;i<p0.nedges;i++) {
		e=reverseIndex[i];

		if (reverseEdges[e].p1!=n) {
			reverseEdgeList[n].n=c;
			n=reverseEdges[e].p1;
			reverseEdgeList[n].start=i;
			c=0;
		}
		c++;
	}
	reverseEdgeList[n].n=c;
}
/****************************************************************************/
int Graph::order(void)
{
	int i;
	double dang,d;
	int c,p,e,n;
	double rad=2000.0;

	while(FOREVER) {
		// Solve shortest path problem
		n=solveShortestPath(0);
		if (n==OK) break;

		// If we failed because of a negative loop add something an edge of the loop
		if (n==NEGATIVE_CYCLE && p0.negativeEdgeCosts) {
			// Find a bad edges
			while(FOREVER) {
				e=checkForNegativeCycle(labels,edges,p0.nedges);
				if (e<0) break;
				else edges[e].cost+=100.0F;
			}
			n=solveShortestPath(0);
			if (n==OK) break;
		}
		else return(NO_CHANGE);
	}

	if (labels[endNode].cost==INF) return(NO_CHANGE);

	// Assert npath<=p0.npts;
	//assert(shortestPath.nlist>0);
	if (shortestPath.nList<p0.npts) dang=180.0/(p0.npts-shortestPath.nList);

	// Draw in the 1000x1000 space
	c=0;
	d=rad/(shortestPath.nList);
	for (i=0;i<p0.npts;i++) {
		p=labels[i].path;
		if (p>=0) {
			nodes[i].x=(-rad/2+d*p);
			nodes[i].y=0.0;
		}
		else {
			c++;
			nodes[i].x=rad*cos(dang*c*DEG_TO_RAD)/2;
			nodes[i].y=rad*sin(dang*c*DEG_TO_RAD)/2;
		}
	}

	ordered=TRUE;
	return(OK);
}
/****************************************************************************/
DataParameters Graph::getParameters(void)
{
	return(p0);
}
/***************************************************************************/
void Graph::setParameters(DataParameters p)
{
	freeAll();
	p0=p;
}
/***************************************************************************/
void Graph::sortPoints(int (*cmp)(void *, void *))
{
	generic_quick_sort(nodes,0,p0.npts-1,sizeof(double)*2,cmp);
}
/***************************************************************************/
int Graph::checkPtsExist(void)
{
	if (nodes==NULL) return(FALSE);
	else return(TRUE);
}
/***************************************************************************/
// Need to test for the presence of a negative cycle since it screws up 
// the shortest path
// Returns one of the offending edges or -1 if no negative cycle exists
int Graph::checkForNegativeCycle(NodeLabels *labels, Edge *edges,int nedges)
{
	int e,u,v;

	for (e=0;e<nedges;e++) {
		u=edges[e].p1;
		v=edges[e].p2;
		if (labels[v].cost>(labels[u].cost+edges[e].cost)) {
			return(e);
		}
	}
	return(-1);
}
/***************************************************************************/
Edge Graph::getEdge(int n)
{
	Edge null_edge(-1,-1,0.0);

	if (n>=0 && n<=p0.nedges) return(edges[n]);
	else return(null_edge);
}
/***************************************************************************/
Edge Graph::getReverseEdge(int n)
{
	Edge null_edge(-1,-1,0.0);

	if (n>=0 && n<=p0.nedges) return(reverseEdges[n]);
	else return(null_edge);
}
/***************************************************************************/
void Graph::setEdge(int n, Edge edgeIn)
{
	if (n>=0 && n<p0.nedges &&
		edgeIn.p1>=0 && edgeIn.p1<p0.npts && 
		edgeIn.p2>=0 && edgeIn.p2<p0.npts) {
		edges[n]=edgeIn;
	}
}
/***************************************************************************/
void Graph::setEdges(Edge *edges_in)
{
	edges=edges_in;
}
/***************************************************************************/
int Graph::findEdge(int p1, int p2)
{
	int hi,lo,guess,node,e;

	lo=forwardEdgeList[p1].start;
	hi=forwardEdgeList[p1].start+forwardEdgeList[p1].n;
	guess=(hi+lo)/2;

	while(FOREVER) {
		e=forwardIndex[guess];
		node=edges[e].p2;
		if (node==p2) return(guess);
		else if (node>p2) hi=guess;
		else lo=guess;
		if (hi==lo) break;

		// Make sure we don't get stuck
		if (hi==lo+1) {
			if (edges[forwardIndex[hi]].p2==p2) return(hi);
			if (edges[forwardIndex[lo]].p2==p2) return(lo);
			else break;	// Nowhere to go
		}
		else guess=(hi+lo)/2;
	}
	return(-1);
}
/***************************************************************************/
// This routines compute block and minblock number information
// It assume that shortest path information has already been filled in
int Graph::computeBlock(void)
{
	int i,j,n,d,b,e,n2;

	// Initialize block and minblock
	for (i=0;i<p0.npts;i++) {
		labels[i].block=(-1);
		labels[i].minblock=(-1);
	}

	// Find block
	for (i=0;i<p0.npts;i++) {
		// If we have not found the block for this node
		if (labels[i].block<0) {
			// If this node is on shortest path we are done
			if (labels[i].path>=0) 
				labels[i].block=labels[i].path;
			else {	// Search
				n=i;
				b=(-1);

				// Push onto stack
				d=0;
				scratchLabels2[d].path=n;	
				d++;

				while(FOREVER) {
					e=labels[n].predEdge;

					// If no connection block=(-1)
					if (e<0) {
						for (j=0;j<d;j++) {
							n=scratchLabels2[j].path;
							labels[n].block=(-1);
						}
						break;
					}
					n=edges[e].p1;
					if (labels[n].block>=0) 
						b=labels[n].block;
					else if (labels[n].path>=0) 
						b=labels[n].path;
					else {	
						scratchLabels2[d].path=n;	// Push onto stack
						d++;
					}

					if (b>=0) {
						for (j=0;j<d;j++) {			// Set all nodes on the stack to same block
							n=scratchLabels2[j].path;
							labels[n].block=b;
						}
						break;
					}
				}
			}
		}
	}

	// Find minblock
	for (i=0;i<p0.npts;i++) {
		// If we have not found the block for this node
		if (labels[i].minblock<0) {
			// If this node is on shortest path we are done
			if (labels[i].path>=0) 
				labels[i].minblock=labels[i].block;
			else {	// Search
				n=i;
				b=(-1);

				// Push onto stack
				d=0;
				scratchLabels2[d].path=n;	
				d++;

				while(FOREVER) {
					e=labels[n].reversePredEdge;

					// If no connection block=(-1)
					if (e<0) {
						for (j=0;j<d;j++) {
							n=scratchLabels2[j].path;
							labels[n].minblock=(-1);
						}
						break;
					}

					n=edges[e].p2;
					if (labels[n].minblock>=0)  
						b=labels[n].minblock;
					else if (labels[n].path>=0) 
						b=labels[n].path;
					else {	// Push onto stack
						scratchLabels2[d].path=n;
						d++;
					}

					if (b>=0) {
						// Unwind
						for (j=d-1;j>=0;j--) {
							n2=scratchLabels2[j].path;
							if (labels[n2].block<b) b=labels[n2].block;	// Find the minimum block
							labels[n2].minblock=b;
						}
						break;
					}
				}
			}
		}
	}
	return(OK);
}
/***************************************************************************/
// This returns the number of nodes in the shortest path
int Graph::getNpath(void)
{
	return(shortestPath.nList+1);
}
/***************************************************************************/
// Returns the position of path of node n or (-1) if not on path
int Graph::getPath(int n)
{
	return(labels[n].path);
}
/***************************************************************************/
// Returns the block of node n
int Graph::getBlock(int n)
{
	return(labels[n].block);
}
/***************************************************************************/
// Returns the minblock of node n
int Graph::getMinBlock(int n)
{
	return(labels[n].minblock);
}
/***************************************************************************/
// Returns the minblock of node n
double Graph::getCost(int n)
{
	return(labels[n].cost);
}
/***************************************************************************/
// Returns the minblock of node n
double Graph::getCostToY(int n)
{
	return(labels[n].reverseCost);
}
/***************************************************************************/
// This routine computes the path structure 
// from start node to n1 to n2 to end node
// Note: If anything went wrong with the sp computation this routine may loop indefinitely
int Graph::computePath(int replacementEdgeIndex, Path &path)
{
	int n,nList1,nList2,c;
	int e;
	Edge edge,replacementEdge;

	// First compute size
	// Compute from p1 back to start node
	replacementEdge=edges[replacementEdgeIndex];
	n=replacementEdge.p1;
	nList1=0;
	while(FOREVER) {
		if (n==startNode) break;
		nList1++;
		n=edges[labels[n].predEdge].p1;
	}

	// Compute from n2 to end node
	n=replacementEdge.p2;
	nList2=0;
	while(FOREVER) {
		if (n==endNode) break;
		nList2++;
		n=edges[labels[n].reversePredEdge].p2;
	}

	path.setSize(nList1+nList2+1);
	path.nList=nList1+nList2+1;
	path.cost=0.0;

	// Now do it again and fill in information
	// Compute from n1 back to start node
	n=replacementEdge.p1;
	c=0;
	while(FOREVER) {
		if (n==startNode) break;
		e=labels[n].predEdge;
		edge=getEdge(e);

		path.list[nList1-c-1]=e;
		path.cost+=edge.cost;
		c++;
		n=edge.p1;
	}

	// This is the key edge
	path.list[nList1]=replacementEdgeIndex;
	path.cost+=replacementEdge.cost;

	// Compute from n2 to end node
	n=replacementEdge.p2;
	c=0;
	while(FOREVER) {
		if (n==endNode) break;
		e=labels[n].reversePredEdge;
		edge=getEdge(e);

		path.list[nList1+c+1]=e;
		path.cost+=edge.cost;

		c++;
		n=edge.p2;
	}

	return(OK);
}
/***************************************************************************/
// Give a copy of the shortest path
void Graph::getShortestPath(Path &path)
{
	path=shortestPath;
}
/***************************************************************************/
int Graph::getStartNode(void)
{
	return(startNode);
}
/***************************************************************************/
int Graph::getEndNode(void)
{
	return(endNode);
}
/***************************************************************************/
void Graph::setStartNode(int n)
{
	startNode=n;
}
/***************************************************************************/
void Graph::setEndNode(int n)
{
	endNode=n;
}
/***************************************************************************/
EdgeList Graph::getEdgeList(int node)
{
	return(forwardEdgeList[node]);
}
/***************************************************************************/
EdgeList *Graph::getEdgeList(void)
{
	return(forwardEdgeList);
}
/***************************************************************************/
NodeLabels *Graph::getLabels(void)
{
	return(labels);
}
/***************************************************************************/
void Graph::setLabels(NodeLabels *labelsIn)
{
	labels=labelsIn;
}
/***************************************************************************/
void Graph::setHybridNumber(int n)
{
	hybridNumber=n;
}
/***************************************************************************/
// Checks that we do not have two edges with the same source and destination
// Warning:Slow computation
void Graph::checkForDuplicateEdge(void)
{
	int i,j;
	char msg[128];

	if (edges==NULL) return;
	for (i=0;i<p0.nedges;i++) {
		for (j=0;j<p0.nedges;j++) {
			if (i==j) continue;
			if (edges[i].p1==edges[j].p1 && 
				edges[i].p2==edges[j].p2) {
				sprintf(msg,"Error:Edge %1d is identical to edge %1d",i,j);
				put_error(msg);
			}
		}
	}
}
/***************************************************************************/
int *Graph::getForwardIndex(void)
{
	return(forwardIndex);
}
/***************************************************************************/
int *Graph::getReverseIndex(void)
{
	return(reverseIndex);
}
/***************************************************************************/
EdgeList *Graph::getForwardEdgeList(void)
{
	return(forwardEdgeList);
}
/***************************************************************************/
EdgeList *Graph::getReverseEdgeList(void)
{
	return(reverseEdgeList);
}
/***************************************************************************/
void Graph::setForwardIndex(int *forwardIndexIn)
{
	forwardIndex=forwardIndexIn;
}
/***************************************************************************/
void Graph::setReverseIndex(int *reverseIndexIn)
{
	reverseIndex=reverseIndexIn;
}
/***************************************************************************/
void Graph::setForwardEdgeList(EdgeList *forwardEdgeListIn)
{
	forwardEdgeList=forwardEdgeListIn;
}
/***************************************************************************/
void Graph::setReverseEdgeList(EdgeList *reverseEdgeListIn)
{
	reverseEdgeList=reverseEdgeListIn;
}
/***************************************************************************/
void Graph::setNodes(Node *nodesIn)
{
	nodes=nodesIn;
}
/***************************************************************************/
// Computes the shortest path
// This assumes the labels have been calculated
int Graph::computePath(NodeLabels *labels, int endNode0, Path &path)
{
	int i;
	int n,e,c,nedges;
	Edge edge;

	if (labels==NULL) return(NOT_FOUND);

	// No path
	if (labels[endNode0].predEdge==(-1)) {
		path.nList=0;
		path.cost=DBL_MAX;
		path.list=NULL;
		return(NOT_FOUND);
	}

	// First count the number of edges on path
	n=endNode0;
	nedges=0;
	while(FOREVER) {
		e=labels[n].predEdge;
		if (e<0) break;
		n=edges[e].p1;
		nedges++;
	}

	// Allocate memory for path
	path.setSize(nedges);
	path.nList=nedges;
	path.cost=labels[endNode0].cost;

	// Now put the edges in order
	n=endNode0;
	c=0;
	while(FOREVER) {
		e=labels[n].predEdge;
		if (e<0) break;				// Check if we have 

		path.list[nedges-c-1]=e;	// Keep track of pred edge
		assert((nedges-c-1)>=0);

		n=edges[e].p1;
		c++;
	}

	// Compute the path for each node
	for (i=0;i<p0.npts;i++) labels[i].path=(-1);
	for (i=0;i<path.nList;i++) {
		edge=getEdge(path.list[i]);
		labels[edge.p1].path=i;
	}
	labels[edge.p2].path=i;

	return(OK);
}
/***************************************************************************/
// Checks validity of the path
// 1. Number of nodes >= 0 
// 2. Paths starts with start node of graph
// 3. Path ends with the end node of graph
// 4. Path is simple that is that no nodes are repeated
int Graph::checkPath(const Path& path)
{
	int i,j;
	char msg[128];
	int e;

	if (path.nList<0) {
		put_error("Number of edges is negative");
		if (Sys.getLogging()) {
			sprintf(msg,"Number of edges is negative:%d\n",path.nList);
			Sys.logString(msg);
			Sys.stopLogging();
		}
		return(TOO_SMALL);
	}

	if (path.nList==0) return(OK);

	for (i=0;i<path.nList;i++) {
		e=path.list[i];
		if (e<0 || e>=p0.nedges) {
			put_error("Bad edge in path");
			if (Sys.getLogging()) {
				sprintf(msg,"Bad edge in path:%1d",e);
				Sys.logString(msg);
				logPath(path);
				Sys.stopLogging();
			}
			return(NO_PATH);
		}
	}
	
	if (edges[path.list[0]].p1!=startNode) {
		put_error("Path does not start with start node for graph");
		if (Sys.getLogging()) {
			sprintf(msg,"Path does not start with start node for graph:%1d",startNode);
			Sys.logString(msg);
			logPath(path);
			Sys.stopLogging();
		}
		return(NO_PATH);
	}

	if (edges[path.list[path.nList-1]].p2!=endNode) {
		put_error("Path does not end with end node for graph");
		if (Sys.getLogging()) {
			sprintf(msg,"Path does not end with end node for graph:%1d",endNode);
			Sys.logString(msg);
			logPath(path);
			Sys.stopLogging();
		}
		return(NO_PATH);
	}

	for (i=0;i<path.nList;i++) {
		for (j=0;j<path.nList;j++) {
			if (i==j) continue;
			if (edges[path.list[i]].p1==edges[path.list[j]].p1) {
				sprintf(msg,"Error:Duplicate point:%d in path found",edges[path.list[i]].p1);
				if (Sys.getLogging()) {
					sprintf(msg,"Error:Duplicate point in path found:%1d ",edges[path.list[i]].p1);
					Sys.logString(msg);
					logPath(path);
					Sys.stopLogging();
				}
				return(NO_PATH);
			}
		}
		if (i!=path.nList-1 &&
			edges[path.list[i]].p2==edges[path.list[path.nList-1]].p2) {
			put_error("Error:Duplicate point in path found");
			if (Sys.getLogging()) {
				sprintf(msg,"Error:Duplicate point in path found:%1d ",edges[path.list[i]].p2);
				Sys.logString(msg);
				logPath(path);
				Sys.stopLogging();
			}
			return(NO_PATH);
		}
	}
	return(OK);
}
/***************************************************************************/
double Graph::calculatePathCost(const Path &path)
{
	int i;
	double cost;

	cost=0.0;
	for (i=0;i<path.nList;i++) 
		cost+=edges[path.list[i]].cost;

	return(cost);
}
/***************************************************************************/
void Graph::formatPath(const Path& p, char *msg)
{
	std::ostringstream m;
	std::string s;
	Path p2;

	edgeToNode(p,p2);
	m << p2;
	s=m.str();
	strcpy(msg,s.data());
}
/***************************************************************************/
void Graph::setReverseEdges(Edge *reverseEdgesIn)
{
	reverseEdges=reverseEdgesIn;
}
/***************************************************************************/
void Graph::setEdgeKey(EdgeKey *edgeKeyIn)
{
	edgeKey=edgeKeyIn;
}
/***************************************************************************/
void Graph::setScratchLabels(NodeLabels *scratchIn)
{
	scratchLabels=scratchIn;
}
/***************************************************************************/
void Graph::setScratchLabels2(NodeLabels *scratchIn)
{
	scratchLabels2=scratchIn;
}
/***************************************************************************/
void Graph::copy(Graph *H)
{
	int i;

	H->setParameters(p0);
	H->setEndNode(endNode);
	H->setForwardIndex(forwardIndex);
	H->setReverseIndex(reverseIndex);
	H->setForwardEdgeList(forwardEdgeList);
	H->setReverseEdgeList(reverseEdgeList);
	H->setNodes(nodes);

	// Copy the edges to scratch
	memcpy(scratchEdges,edges,p0.nedges*sizeof(Edge));

	H->setLabels(scratchLabels);
	H->setScratchLabels2(scratchLabels2);

	for (i=0;i<p0.npts;i++) 
		scratchLabels[i].valid=TRUE;

	H->setEdges(scratchEdges);
	H->setReverseEdges(reverseEdges);
	H->setEdgeKey(edgeKey);
}
/***************************************************************************/
void Graph::release(void)
{
	forwardIndex=NULL;
	reverseIndex=NULL;
	forwardEdgeList=NULL;
	reverseEdgeList=NULL;

	scratchEdges=NULL;
	scratchLabels=NULL;
	scratchLabels2=NULL;

	labels=NULL;
	edges=NULL;
	reverseEdges=NULL;
	edgeKey=NULL;
	nodes=NULL;
}
/***************************************************************************/
void Graph::setEdgeInvalid(int e)
{
	edges[e].cost=INF;
}
/***************************************************************************/
void Graph::setNodeInvalid(int n)
{
	labels[n].valid=FALSE;
}
/***************************************************************************/
int Graph::getNumBadCases(void)
{
	return(numBadCases);
}
/***************************************************************************/
void Graph::resetNumBadCases(void)
{
	numBadCases=0;
}
/***************************************************************************/
int Graph::getNumDijkstras(void)
{
	return(numDijkstras);
}
/***************************************************************************/
void Graph::resetNumDijkstras(void)
{
	numDijkstras=0;
}
/***************************************************************************/
int Graph::getNumBranchPaths(void)
{
	return(numBranchPaths);
}
/***************************************************************************/
void Graph::resetNumBranchPaths(void)
{
	numBranchPaths=0;
}
/***************************************************************************/
int Graph::getTotalBranchLengths(void)
{
	return(totalBranchLengths);
}
/***************************************************************************/
void Graph::resetTotalBranchLengths(void)
{
	totalBranchLengths=0;
}
/***************************************************************************/
int Graph::getNumKSP(void)
{
	return(numKSP);
}
/***************************************************************************/
void Graph::resetNumKSP(void)
{
	numKSP=0;
}
/***************************************************************************/
int Graph::getTotalKSPLengths(void)
{
	return(totalKSPLengths);
}
/***************************************************************************/
void Graph::resetTotalKSPLengths(void)
{
	totalKSPLengths=0;
}
/***************************************************************************/
int Graph::getNumNodesScanned(void)
{
	return(numNodesScanned);
}
/***************************************************************************/
void Graph::resetNumNodesScanned(void)
{
	numNodesScanned=0;
}
/***************************************************************************/
int Graph::getNumNodes(void)
{
	return(p0.npts);
}
/***************************************************************************/
// This turns a path which represents edges to a path that represents nodes
void Graph::edgeToNode(const Path &p, Path &p2)
{
	int i;

	if (p.nList==0) {
		p2.nList=0;
		p2.cost=0.0;
	}
	else {
		p2.setSize(p.nList+1);
		for (i=0;i<p.nList;i++) 
			p2.list[i]=edges[p.list[i]].p1;
		p2.list[p.nList]=edges[p.list[p.nList-1]].p2;
		p2.nList=p.nList+1;
		p2.cost=p.cost;
	}
}
/***************************************************************************/
int  fastCompare(const void *p1, const void *p2)
{
	Edge e1,e2;
	int *i1,*i2;
	i1=(int *)p1;
	i2=(int *)p2;
	e1=current_graph->getEdge(*i1);
	e2=current_graph->getEdge(*i2);
	if (e1.p1>e2.p1) return(1);
	else if (e1.p1<e2.p1) return(-1);
	else if (e1.p2>e2.p2) return(1);
	else if (e1.p2<e2.p2) return(-1);
	else return(0);
}
/***************************************************************************/
int  fastRCompare(const void *p1, const void *p2)
{
	Edge e1,e2;
	int *i1,*i2;
	i1=(int *)p1;
	i2=(int *)p2;
	e1=current_graph->getReverseEdge(*i1);
	e2=current_graph->getReverseEdge(*i2);
	if (e1.p1>e2.p1) return(1);
	else if (e1.p1<e2.p1) return(-1);
	else if (e1.p2>e2.p2) return(1);
	else if (e1.p2<e2.p2) return(-1);
	else return(0);
}
/***************************************************************************/
int  fastCostCompare(const void *p1, const void *p2)
{
	Edge e1,e2;
	int *i1,*i2;
	i1=(int *)p1;
	i2=(int *)p2;
	e1=current_graph->getEdge(*i1);
	e2=current_graph->getEdge(*i2);
	if (e1.cost>e2.cost) return(1);
	else if (e1.cost<e2.cost) return(-1);
	else return(0);
}
/***************************************************************************/
double Graph::getPtVal(int n, int d)
{
	if (n>=0 && n<p0.npts && d>=0 && d<2) {
		if (d==0) return(nodes[n].x);
		else return(nodes[n].y);
	}
	else return(0.0);
}
/***************************************************************************/
