/******************************************************************************/
/*
	Graph.cpp
	Created Oct, 2001
*/
/******************************************************************************/
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <math.h>
#include <assert.h>
#include <cstdlib>
#include <map>
/******************************************************************************/
#include "Graph.h"
#include "BranchingStructure.h"
#include "FibonacciHeap.h"
#include "NodeLabels.h"
#include "Path.h"
#include "LinkList.h"
#include "DataParam.h"
#include "Node.h"
/******************************************************************************/
#include "const.h"
#include "xstats.h"
#include "xmenus.h"
#include "xutil.h"
#include "xdrw.h"
#include "xgraphic.h"
#include "xprim.h"
#include "xcompare.h"
#include "colors.h"
#include "ret.h"
#include "xrout.h"
/******************************************************************************/
using std::ofstream;
using std::ifstream;
/******************************************************************************/
void Graph::createDrawingForPts(void)
{
	int i;

	if (nodes==NULL) return;

	// Create the visual
	// Delete old drawing if we have one
//	drw_del_name("Data Drawing");
//	drw_open_drw("Data Drawing");
//	drw_open_seg(GRAY,MARKER,11);

	if (p0.npts<10000) {
		// Project to screen
		for (i=0;i<p0.npts;i++) 
			createDrawingForPt(i);
	}
	else {
		createDrawingForPt(startNode);
		createDrawingForPt(endNode);
	}
}
/******************************************************************************/
void Graph::createDrawingForPt(int n)
{
	double x0,y0;
					  
	if (n<0 || n>=p0.npts) return;
//	drw_open_spt(nodes[n].x,nodes[n].y);
	if (Sys.getLabelPoints()) {
		char msg[128];
		sprintf(msg,"%1d",n);
		xyz_to_geo(nodes[n].x,nodes[n].y,&x0,&y0);
//		drw_open_lab(x0,y0,WHITE,msg,-13,"",LAB_RIGHT|LAB_TOP);
		if (Sys.getShowNodeLabels()) {
			if (labels!=NULL && labelsValid==TRUE) {
				int pred,rpred;
				if (labels[n].predEdge<0 || labels[n].predEdge>=p0.nedges) 
					pred=(-1);
				else pred=edges[labels[n].predEdge].p1;

				if (labels[n].reversePredEdge<0 || labels[n].reversePredEdge>=p0.nedges)
					rpred=(-1);
				else rpred=edges[labels[n].reversePredEdge].p2;

				if (labels[n].cost<INF) 
					sprintf(msg,"Cost:%1.1lf Pre:%1d B:%1d RPre:%1d M:%1d",
						labels[n].cost,pred,labels[n].block,rpred,labels[n].minblock);
				else 
					sprintf(msg,"Cost:INF Pre:%1d B:%1d RPre:%1d M:%1d",
						pred,labels[n].block,rpred,labels[n].minblock);

				xyz_to_geo(nodes[n].x,nodes[n].y,&x0,&y0);
//				drw_open_lab(x0,y0,WHITE,msg,-13,"",LAB_RIGHT|LAB_BOTTOM);
			}
			else {
				xyz_to_geo(nodes[n].x,nodes[n].y,&x0,&y0);
//				drw_open_lab(x0,y0,WHITE,"Not Valid",-13,"",LAB_RIGHT|LAB_BOTTOM);
			}
		}
	}
}
/****************************************************************************/
void Graph::createDrawingForLabels(void)
{
	int i;
	double x0,y0;
	char msg[128];

	// Create the visual
	// Delete old drawing if we have one
//	drw_del_name("Label Info");
//	drw_open_drw("Label Info"); // Open a new drawing
//	drw_open_seg(GRAY,MARKER,11);

	// Project to screen
	for (i=0;i<p0.npts;i++) 
//		drw_open_spt(nodes[i].x,nodes[i].y);

	if (Sys.getLabelEdges()) {
		for (i=0;i<p0.npts;i++) {
			if (labels[i].cost==INF) {
				sprintf(msg,"%1d(c:%Inf,b:%1d,m:%1d)",i,
					labels[i].block,
					labels[i].minblock);
			}
			else {
				sprintf(msg,"%1d(c:%5.1f,b:%1d,m:%1d)",i,
					labels[i].cost,
					labels[i].block,
					labels[i].minblock);
			}
			xyz_to_geo(nodes[i].x,nodes[i].y,&x0,&y0);
//			drw_open_lab(x0,y0,WHITE,msg,-13,"",LAB_RIGHT|LAB_TOP);
		}
	}
}
/****************************************************************************/
void Graph::createDrawing(void) 
{
	int i;
	long c=mapColor(0);
	double ax,ay;

	if (p0.npts<=0 || nodes==NULL) return;

	createDrawingForPts();

	xyz_to_geo(nodes[startNode].x,nodes[startNode].y,&ax,&ay);
//	drw_open_lab(ax,ay,c,"S",-13,"",LAB_LEFT|LAB_TOP);

	xyz_to_geo(nodes[endNode].x,nodes[endNode].y,&ax,&ay);
//	drw_open_lab(ax,ay,c,"T",-13,"",LAB_LEFT|LAB_TOP);

	if (p0.nedges<10000) {
		//for (i=0;i<p0.nedges;i++) 
//			createDrawingForEdge(edges[i],DARK_GRAY,SOLID,0.0,0.0,TRUE);
	}

	if (shortestPath.nList) {
		createDrawingForPath(shortestPath,"Shortest Path",c,0.0,20.0);
	}
}
/****************************************************************************/
// Draw the edge based on the index for edge
void Graph::createDrawingForEdge(int n, int color, int linetype, 
	double x_offset, double y_offset, int text_flag)
{
	if (n>=0 && n<p0.nedges)
		createDrawingForEdge(edges[n],color,linetype,x_offset,y_offset,text_flag);
}
/****************************************************************************/
void Graph::createDrawingForEdge(Edge &e, int color, int linetype, 
	double x_offset, double y_offset, int text_flag)
{
	double x,y,x1,x2,y1,y2,a,x0,y0;
	int xp,yp,xp1,yp1,xp2,yp2,jump=FALSE;
	double a1,a2,tmp,r;
	double lat,lon;
	char msg[128];
	int first,swap=FALSE;
	int dist=1;
	int arrowLen=30;

	// Draw the edges
//	drw_open_seg(color,LINE,linetype);
	x1=nodes[e.p1].x;
	y1=nodes[e.p1].y;
	x2=nodes[e.p2].x;
	y2=nodes[e.p2].y;

	// Find out if we jump
	if (labels!=NULL) {
		// If both ends are on path and distance between them > 1
		dist=abs(labels[e.p1].path-labels[e.p2].path);
		if (labels[e.p1].path>=0 && 
			labels[e.p2].path>=0 && 
			dist>1) jump=TRUE;
	}

	if (jump && ordered) {
		// Number in of jumps in path
		if (shortestPath.nList==0) r=2000.0;
		else r=2000*dist/(shortestPath.nList);
		x0=(x1+x2)/2;
		y0=r*0.8660254;
		a1=RAD_TO_DEG*atan2((y2-y0),(x2-x0));
		if (a1<0.0) a1+=360.0;
		a2=RAD_TO_DEG*atan2((y1-y0),(x1-x0));
		if (a2<0.0) a2+=360.0;
		if (a1>a2) {
			tmp=a1;
			a1=a2;
			a2=tmp;
			swap=TRUE;
		}
		first=TRUE;
		for (a=a1;a<a2;a+=2.0) {
			x=r*cos(a*DEG_TO_RAD)+x0;
			y=r*sin(a*DEG_TO_RAD)+y0;
//			drw_open_spt(x,y+y_offset);
		}
		x=r*cos(a2*DEG_TO_RAD)+x0;
		y=r*sin(a2*DEG_TO_RAD)+y0;
		//drw_open_spt(x,y+y_offset);

		if (swap) a=(a1+a2*2)/3;
		else a=(a1*2+a2)/3;
		if (text_flag) {
			x=r*cos(a*DEG_TO_RAD)+x0;
			y=r*sin(a*DEG_TO_RAD)+y0;
			sprintf(msg,"%1.2f",e.cost);
			xyz_to_geo(x,y,&lat,&lon);
//			drw_open_lab(lat,lon,GRAY,msg,-13,"",LAB_CENTER|LAB_MIDDLE);
		}

		// Draw arrow heads
		if (p0.directed) {
			// From p1 to p2
			if (swap) a=(a2*19+a1)/20;
			else a=(a1*19+a2)/20;

			xp=r*cos(a*DEG_TO_RAD)+x0;
			yp=r*sin(a*DEG_TO_RAD)+y0;

			if (swap) {
				xp1=xp+(int)(arrowLen*cos((150.0+30.0)*DEG_TO_RAD));
				yp1=yp-(int)(arrowLen*sin((150.0+30.0)*DEG_TO_RAD));
			}
			else {
				xp1=xp+(int)(arrowLen*cos((30.0+30.0)*DEG_TO_RAD));
				yp1=yp-(int)(arrowLen*sin((30.0+30.0)*DEG_TO_RAD));				
			}
			if (swap) {
				xp2=xp+(int)(arrowLen*cos((150.0-30.0)*DEG_TO_RAD));
				yp2=yp-(int)(arrowLen*sin((150.0-30.0)*DEG_TO_RAD));
			}
			else {
				xp2=xp+(int)(arrowLen*cos((30.0-30.0)*DEG_TO_RAD));
				yp2=yp-(int)(arrowLen*sin((30.0-30.0)*DEG_TO_RAD));
			}
//			drw_open_seg(color,LINE,SOLID);
//			drw_open_spt(xp1,yp1+y_offset);
//			drw_open_spt(xp,yp+y_offset);
//			drw_open_spt(xp2,yp2+y_offset);
		}
	}
	else {
//		drw_open_spt(x1,y1+y_offset);
//		drw_open_spt(x2,y2+y_offset);
		if (Sys.getLabelEdges() && text_flag) {
			sprintf(msg,"%1.2f",e.cost);
			x=(nodes[e.p1].x+3*nodes[e.p2].x)/4;
			y=(nodes[e.p1].y+3*nodes[e.p2].y)/4;
			xyz_to_geo(x,y+y_offset,&lat,&lon);
//			drw_open_lab(lat,lon,GRAY,msg,-13,"",LAB_CENTER|LAB_MIDDLE);
		}

		// Draw arrow heads
		if (p0.directed) {
			// From p1 to p2
			a=atan2((y1-y2),(x1-x2));
			x0=(x1+x2*19)/20;
			y0=(y1+y2*19)/20;
			xyz_to_pix(x0,y0,&xp,&yp);

			xp1=xp+(int)(8*cos(a+20.0*DEG_TO_RAD));
			yp1=yp-(int)(8*sin(a+20.0*DEG_TO_RAD));
			pix_to_xyz(xp1,yp1,&x,&y);
//			drw_open_seg(color,LINE,SOLID);
//			drw_open_spt(x,y+y_offset);
//			drw_open_spt(x0,y0+y_offset);

			xp2=xp+(int)(8*cos(a-20.0*DEG_TO_RAD));
			yp2=yp-(int)(8*sin(a-20.0*DEG_TO_RAD));
			pix_to_xyz(xp2,yp2,&x,&y);
//			drw_open_seg(color,LINE,SOLID);
//			drw_open_spt(x,y+y_offset);
//			drw_open_spt(x0,y0+y_offset);
		}
	}
}
/****************************************************************************/
void Graph::createDrawingForShortestPath(void)
{
	if (shortestPath.nList) {
		createDrawingForPath(shortestPath,"Shortest Path",GREEN,0.0,20.0);
	}
}
/***************************************************************************/
void Graph::createDrawingForPath(const Path &path, char *name, int color, 
	double x_offset, double y_offset) 
{
	double ax,ay;
	char msg[1024];
	double rad=2000.0;
	int i;
	Edge edge;

	xyz_to_geo(-rad/2+x_offset,-400+y_offset,&ax,&ay);
	if (path.nList==0) {
		sprintf(msg,"%s not found",name);
//		drw_open_lab(ax,ay,color,msg,-13,"",LAB_RIGHT|LAB_BOTTOM);
	}
	else {
		formatPath(name,path,msg);
//		drw_open_lab(ax,ay,color,msg,-13,"",LAB_RIGHT|LAB_BOTTOM);

		for (i=0;i<path.nList;i++) {
			edge=getEdge(path.list[i]);
//			createDrawingForEdge(edge,color,SOLID,x_offset/10,y_offset/10,0);
		}
	}
}
/***************************************************************************/
long Graph::mapColor(int n)
{
	int r,g,b,color,k=31+n*5;
	r=(k%4)*64;
	g=((k%16)/4)*64;
	b=((k%64)/16)*64;
	color=(b<<16)+(g<<8)+r;
	if (color==0) color=0x202020;	// Dark Gray
	return(color);
}
/***************************************************************************/
void Graph::deleteDrawings(void)
{
//	drw_del_name("Data Drawing");
//	drw_del_name("Shortest Path");
//	drw_del_name("Label Info");
//	drw_del_name("Replacement Paths");
//	drw_del_name("New KSP");
//	drw_del_name("Yen KSP");
//	drw_del_name("Edge labels");
//	drw_del_name("Y-sp tree");
//	drw_del_name("X-sp tree");
}
/***************************************************************************/

