/******************************************************************************/
#include <string.h>
#include <sstream>
#include <math.h>
#include <fstream>
#include <cstdlib>
#include <assert.h>
#include<iostream>
/******************************************************************************/
#include "Graph.h"
#include "xmenus.h"
#include "xrout.h"
#include "ret.h"
#include "const.h"
#include "xutil.h"
/******************************************************************************/
using std::ofstream;
using std::ifstream;
using namespace std;
/******************************************************************************/
// This generates a graph based on the input parameters p
void Graph::generate(DataParameters p)
{
	int i;
	int showInfo=Sys.getShowInfo();

	// Delete old drawings - create new drawing
	if (showInfo) show_msg("Deleting drawings...");
	deleteDrawings();

	// Generating new data free everything
	if (showInfo) show_msg("Free old data...");
	freeAll();

	// Check for a bad parameter
	if (p.npts==0) return;

	// Generate space for graph
	genSpace(p);

	// Init some flags
	ordered=FALSE;
	labelsValid=FALSE;

	// Call appropriate generating routine
	if (p.grid==TRUE) gridGenerate(p);
	else randomGenerate(p);

	// Compute the reverse edges
	if (showInfo) show_msg("Computing reverse edges");
	for (i=0;i<p0.nedges;i++) {
		reverseEdges[i].p1=edges[i].p2;
		reverseEdges[i].p2=edges[i].p1;
		reverseEdges[i].cost=edges[i].cost;
	}

	// This structure allows us to find all the edges from or to a node n
	computeEdgeLists();

	// If we are drawing create a drawing
	if (Sys.getShowGraphics()) createDrawing();

	if (showInfo) show_msg("Done with generation...");

}
/******************************************************************************/
// This generates a neighborhood graph on a grid
void Graph::gridGenerate(DataParameters p)
{
	int i,j,w,showInfo,ne;
	int full,dir[8],n,cnt,row,col;
	double dx,dy;

	showInfo=Sys.getShowInfo();

	if (((int)p.nedges/p.npts)>7) full=TRUE;
	else full=FALSE;

	// Arrange points and init labels
	if (showInfo) show_msg("Arranging points...");

	// Width
	w=p.npts/p.length;

	// Slightly adjust the number of points
	p.npts=w*p.length;

	startNode=0;
	endNode=p.npts-1;

	// Arrange points
	dx=1000.0/(p.length-1);
	dy=1000.0/(w-1);

	for (i=0;i<p.npts;i++) {
	
		nodes[i].x=(i%(p.length))*dx-500.0;
		nodes[i].y=(i/(p.length))*dy-500.0;

		// Init basic labels as well
		labels[i].num=i;
		scratchLabels[i].num=i;
		scratchLabels2[i].num=i;
		labels[i].valid=TRUE;
	}


	ne=0;
	for (i=0;i<p.npts;i++) {
		for (j=0;j<8;j++) dir[j]=FALSE;
		if (full) {
			for (j=0;j<8;j++) dir[j]=TRUE;	// All directions
		}
		else {
			while(FOREVER) {
				n=((int)(drnd()*8));
				if (n>=8) n=0;
				dir[n]=TRUE;
				cnt=0;
				for (j=0;j<8;j++) {
					if (dir[j]==TRUE) {
						cnt++;
						if (cnt>=4) break;
					}
				}
				if (cnt>=4) break;
			}
		}

		row=i/p.length;
		col=i%p.length;

		// Down
		if (row>0) {
			// Down
			if (dir[0]) {
				edges[ne].p1=i;
				edges[ne].p2=i-p.length;
				edges[ne].cost=1000.0*drnd();
				ne++;
			}

			// Down-Right
			if (dir[1] && col<(p.length-1)) {
				edges[ne].p1=i;
				edges[ne].p2=i-p.length+1;
				edges[ne].cost=1000.0*drnd();
				ne++;
			}

			// Down-Left
			if (dir[2] && col>0) {
				// Create down-left
				edges[ne].p1=i;
				edges[ne].p2=i-p.length-1;
				edges[ne].cost=1000.0*drnd();
				ne++;
			}
		}

		// Up
		if (row<(w-1)) {
			if (dir[3]) {
				edges[ne].p1=i;
				edges[ne].p2=i+p.length;
				edges[ne].cost=1000.0*drnd();
				ne++;
			}

			// Up-Right
			if (dir[4] && col<(p.length-1)) {
				edges[ne].p1=i;
				edges[ne].p2=i+p.length+1;
				edges[ne].cost=1000.0*drnd();
				ne++;
			}

			// Up-Left
			if (dir[5] && col>0) {
				// Create down-left
				edges[ne].p1=i;
				edges[ne].p2=i+p.length-1;
				edges[ne].cost=1000.0*drnd();
				ne++;
			}
		}

		// Left
		if (dir[6] && col>0) {
			edges[ne].p1=i;
			edges[ne].p2=i-1;
			edges[ne].cost=1000.0*drnd();
			ne++;
		}
		if (dir[7] && col<(p.length-1)) {
			edges[ne].p1=i;
			edges[ne].p2=i+1;
			edges[ne].cost=1000.0*drnd();
			ne++;
		}

	}

	// Modify actual parameters
	p0.nedges=ne;
	p0.npts=p.npts;

}
/******************************************************************************/
// Generate a random graph
void Graph::randomGenerate(DataParameters p)
{
	int i,j,e,*edge_matrix=NULL,node,rp,m,showInfo;
	double ang;
	char msg[128];

	showInfo=Sys.getShowInfo();

	// Arrange points and init labels
	if (showInfo) 
		show_msg("Arranging points...");
	for (i=0;i<p0.npts;i++) {

		// Arrange points in a circle to see all edges
		ang=360.0*i/p.npts;
		nodes[i].x=1000.0*cos(ang*DEG_TO_RAD);
		nodes[i].y=1000.0*sin(ang*DEG_TO_RAD);

		// Init basic labels as well
		labels[i].num=i;
		scratchLabels[i].num=i;
		scratchLabels2[i].num=i;
		labels[i].valid=TRUE;
	}

	
	// For graphs with less than 500 points create a matrix to give us know edges 
	// Probably a better way to do this but
	if (p0.npts<=500) {
		edge_matrix=new int[p0.npts*p0.npts];
		if (edge_matrix==NULL) {
			put_error("Error:No memory to allocate edge matrix");
			return;
		}
		for (i=0;i<p0.npts;i++) {
			for (j=0;j<p0.npts;j++) edge_matrix[i*p0.npts+j]=(-1);
		}
	}

	// Find the start and the end
	startNode=(int)(p0.npts*drnd());
	while(FOREVER) {
		endNode=(int)(p0.npts*drnd());
		if (endNode!=startNode) break;
	}

	// Fully connected ?
	if (p0.nedges==(p0.npts*(p0.npts-1))) {
		e=0;
		for (i=0;i<p0.npts;i++) {
			for (j=i+1;j<p0.npts;j++) {
				edges[e].p1=i;
				edges[e].p2=j;

				if (p0.negativeEdgeCosts) edges[e].cost=2000.0*drnd()-1000.0;
				else edges[e].cost=1000.0*drnd();

				// Always goes bidirected in fully connected network 
				edges[e+1].p1=j;
				edges[e+1].p2=i;

				// If not directed use the same cost
				if (p0.directed==FALSE) edges[e+1].cost=edges[e].cost;
				else {
					if (p0.negativeEdgeCosts) edges[e+1].cost=2000.0*drnd()-1000.0;
					else edges[e+1].cost=1000.0*drnd();
				}

				e+=2;
			}
		}
	}
	else {
		if (showInfo) 
			show_msg("Find edges...");
		if (p0.npts>500) {
			// Distribute edges
			e=0;
			int p=p0.npts*(p0.npts-1);
			double d=((double)p0.nedges)/p0.npts;
			double chance;

			// Make sure a path exists
			int pathlen=2*drnd()*log(p0.npts/2)/log(d);	// Guess a path length
			if (pathlen<2) pathlen=2;
			Path realPath(pathlen);
			realPath.nList=pathlen;
			realPath.list[0]=startNode;
			realPath.list[pathlen-1]=endNode;
			for (i=1;i<pathlen-1;i++) {
				while(FOREVER) {
					node=p0.npts*drnd();
					if (node==endNode) continue;
					int valid=TRUE;
					for (j=0;j<i;j++) {
						if (realPath.list[j]==node) {
							valid=FALSE;
							break;
						}
					}
					if (valid==TRUE) break;
				}
				realPath.list[i]=node;
			}


			for (i=0;i<p0.npts;i++) {
				rp=(-1);
				for (m=0;m<realPath.nList;m++) {
					if (i==realPath.list[m]) {
						rp=m;
						break;
					}
				}
				for (j=0;j<p0.npts;j++) {
					if (i==j) continue;
					if (rp>=0 && realPath.list[rp+1]==j) chance=1.0;
					else chance=((double)(p0.nedges-e))/(p-i*(p0.npts-1)-j-1);
					if (drnd()<chance) {
						edges[e].p1=i;
						edges[e].p2=j;
						edges[e].cost=1000.0*drnd();
						e++;
						if (showInfo && (e%100)==0) {
							sprintf(msg,"Edge %d",e);
							show_msg(msg);
						}
					}
					if (e>=p0.nedges) break;
				}
				if (e>=p0.nedges) break;
			}


		}
		else {
			e=0;
			// Generate the edges
			for (i=0;i<p0.nedges;i++) {
				// Create an edge
				while(FOREVER) {
					// Generate random end points
					edges[e].p1=(int)(p0.npts*drnd());
					edges[e].p2=(int)(p0.npts*drnd());

					// Make sure we are in bounds
					if (edges[e].p1>=p0.npts) edges[e].p1=p0.npts-1;
					if (edges[e].p2>=p0.npts) edges[e].p2=p0.npts-1;

					// Check if this edge already exists
					if (edge_matrix && edge_matrix[edges[e].p1*p0.npts+edges[e].p2]!=(-1)) continue;

					// Check that we are not at the same endpoints
					if (edges[e].p1==edges[e].p2) continue;

					// Edge is ok save it in the list 

					// Generate edge cost
					// If negative costs make the negative costs smaller 
					// to reduce chance of negative cycles
					if (p0.negativeEdgeCosts) edges[e].cost=1100.0*drnd()-100.0;
					else edges[e].cost=1000.0*drnd();

					// Create the edge list entry(s)
					edge_matrix[edges[e].p1*p0.npts+edges[e].p2]=e; // Always goes forward direction
					e++;
					if (e>=p0.nedges) break;	// Check if we have enough edges

					if (p0.directed==FALSE) {
						// If reverse edge does not exist create it
						if (edge_matrix[edges[e-1].p2*p0.npts+edges[e-1].p1]==(-1)) {
							edges[e].p1=edges[e-1].p2;
							edges[e].p2=edges[e-1].p1;
							edges[e].cost=edges[e-1].cost;
							edge_matrix[edges[e].p1*p0.npts+edges[e].p2]=e;
							e++;
							i++;
						}
					}
					break;
				}
			}
		}
	}

	if (edge_matrix) delete edge_matrix;

}
/****************************************************************************/
// This routine allocates space for the graph and scratch space for 
// routines which calculate the k best paths
void Graph::genSpace(DataParameters param)
{
	int i,showInfo;

	showInfo=Sys.getShowInfo();

	// Delete old drawings - create new drawing
	if (showInfo) //show_msg("Deleting drawings...");
	deleteDrawings();

	// Generating new data free everything
	if (showInfo) //show_msg("Free old data...");
	freeAll();

	if (param.npts==0) return;

	p0=param;

	// Allocate space for nodes
	if (showInfo) 
		//show_msg("Allocating space for points...");
	nodes=new Node[param.npts];
	assert(nodes!=NULL);

	// Allocate memory to store the edges 
	if (showInfo) 
		//show_msg("Allocating edges...");
	if (p0.directed==FALSE) p0.nedges=(p0.nedges/2)*2;		// Make sure edges are even if bidirectional
	if (p0.nedges>0) {
		edges=new Edge[p0.nedges];
		assert(edges!=NULL);
	}

	// Create space for labels
	if (showInfo) 
		//show_msg("Allocating node labels...");
	labels=new NodeLabels[p0.npts];
	assert(labels!=NULL);

	if (showInfo) 
		//show_msg("Allocating scratch space for labels...");
	scratchLabels=new NodeLabels[p0.npts];	// Used for computing shortest path in subgraphs
	assert(scratchLabels!=NULL);

	if (showInfo) 
		//show_msg("Allocating 2nd scratch space for labels...");
	scratchLabels2=new NodeLabels[p0.npts];	// Used for computing shortest path in subgraphs
	assert(scratchLabels2!=NULL);

	for (i=0;i<p0.npts;i++) {
		labels[i].num=i;
		scratchLabels[i].num=i;
		scratchLabels2[i].num=i;
		labels[i].valid=TRUE;
	}

	if (showInfo) 
		//show_msg("Allocating scratch space for edges...");
	scratchEdges=new Edge[p0.nedges];		// Used for computing shortest path in subgraphs
	assert(scratchEdges!=NULL);

	if (showInfo) 
		//show_msg("Allocating space for reverse edges for solve destination shortest path...");
	reverseEdges=new Edge[p0.nedges];
	assert(reverseEdges!=NULL);

	if (showInfo) 
		//show_msg("Allocating keys for each edge...");
	edgeKey=new EdgeKey[p0.nedges];
	assert(edgeKey!=NULL);
	return;
}
/****************************************************************************/
void Graph::freeAll(void)
{

	// Main point list
	if (nodes!=NULL) delete [] nodes;
	nodes=NULL;
	p0.npts=0;

	// Free the shortest path
	shortestPath=NULLPATH;

	// Delete the forward node labels
	if (labels!=NULL) delete labels;
	labels=NULL;

	// Delete the list of edges
	if (edges!=NULL) delete [] edges;
	edges=NULL;

	// Delete the scratch
	if (scratchEdges!=NULL) delete [] scratchEdges;
	scratchEdges=NULL;

	// Delete the reverse Edges
	if (reverseEdges!=NULL) delete [] reverseEdges;
	reverseEdges=NULL;

	// Delete the edge list of source nodes
	if (forwardEdgeList!=NULL) delete [] forwardEdgeList;
	forwardEdgeList=NULL;

	// Delete the edge list of destination nodes
	if (reverseEdgeList!=NULL) delete [] reverseEdgeList;
	reverseEdgeList=NULL;

	// Delete the index to the edges sorted by source node
	if (forwardIndex!=NULL) delete [] forwardIndex;
	forwardIndex=NULL;

	// Delete the index to the edges sorted by destination node
	if (reverseIndex!=NULL) delete [] reverseIndex;
	reverseIndex=NULL;

	// Delete the reverse Edges
	if (edgeKey!=NULL) delete [] edgeKey;
	edgeKey=NULL;

	// Delete the 1st scratchLabels
	if (scratchLabels!=NULL) delete [] scratchLabels;
	scratchLabels=NULL;

	// Delete the 2nd scratchLabels
	if (scratchLabels2!=NULL) delete [] scratchLabels2;
	scratchLabels2=NULL;
}
/***************************************************************************/
int Graph::inputRaw(char *path) 
{
	int i,j,n,lineno=0,showInfo;
	char s[1024],msg[1024];
	ifstream fp(path,ios::in);
	int nodeNumbers[65536],nNodes=0;
	int nEdges=0,node,nodeNumber,dest,nc;
	char word[32];

	showInfo=Sys.getShowInfo();

	// If we cannot open file
	if (!fp) {
		sprintf(msg,"Error:Cannot open file for reading:%s",path);
		put_error(msg);
		return(NO_CHANGE);
	}

	// Free all other data
	freeAll();

	// Print a message to the user so he does not get bored
	if (showInfo) {
		sprintf(msg,"Loading graph:%s from file",path);
		show_msg(msg);
	}


	// Scan graph to find the number of nodes and number of edges
	while(FOREVER) {
		fp >> s;
		if (s[0]=='A' && s[1]=='S') {
			sscanf(s,"AS%d",&nodeNumber);

			node=(-1);
			for (i=0;i<nNodes;i++) {
				if (nodeNumbers[i]==node) {
					node=i;
					break;
				}
			}

			if (node==(-1)) {
				node=nNodes;
				nodeNumbers[nNodes]=nodeNumber;
				nNodes++;
			}

		}
		else if ((s[0]=='P' && s[1]=='r') ||
			(s[0]=='C' && s[1]=='u') ||
			(s[0]=='S' && s[1]=='i') ||
			(s[0]=='P' && s[1]=='e')) {
			fp >> s;
			sscanf(s,"#:%d",&n);
			nEdges+=n;
			fp >> s;
		}
		if (fp.eof()==TRUE) break;
	}

	fp.close();
	fp.open(path,ios::in);

	// 2nd Pass
	while(FOREVER) {
		fp >> s;
		if (s[0]=='A' && s[1]=='S') {
			sscanf(s,"AS%d",&nodeNumber);

			node=(-1);
			for (i=0;i<nNodes;i++) {
				if (nodeNumbers[i]==nodeNumber) {
					node=i;
					break;
				}
			}

			if (node==(-1)) {
				put_error("Error:New node found");
			}
		}
		else if ((s[0]=='P' && s[1]=='r') ||
			(s[0]=='C' && s[1]=='u') ||
			(s[0]=='S' && s[1]=='i') ||
			(s[0]=='P' && s[1]=='e')) {
			fp >> s;
			sscanf(s,"#:%d",&n);
			nEdges+=n;
			fp >> s;
			for (i=1;i<n;i++) {
				if (s[i]==':') {
					word[nc]=0;
					nc=0;
					nodeNumber=atoi(word);
					// Find this node
					dest=(-1);
					for (j=0;j<nNodes;j++) {
						if (nodeNumbers[j]==nodeNumber) {
							dest=j;
							break;
						}
					}
					if (dest==(-1)) {
						put_error("Destination node not found");
					}
					else {
						// Add this edge (node,dest)
					}
				}
				else {
					word[nc]=s[i];
					nc++;
				}
			}
		}
		if (fp.eof()==TRUE) break;
	}
	

	return(OK);
}
/***************************************************************************/
int Graph::output(char *path) 
{
	int i;
	char msg[384];
	ofstream fp(path,ios::out);

	if (!fp) {
		sprintf(msg,"Error:Cannot open file for writing:%s",path);
		put_error(msg);
		return(NO_CHANGE);
	}

	// Output a header
	fp << "Graph\n";
	fp << "NumberOfPoints: " << p0.npts << "\n";
	fp << "NumberOfEdges: " << p0.nedges << "\n";
	fp << "StartNode: " << startNode << "\n";
	fp << "EndNode: " << endNode << "\n";

	// Output the points 
	for (i=0;i<p0.npts;i++)
		fp << "Point " << nodes[i].x << " " << nodes[i].y << "\n";

	// Output the edges
	for (i=0;i<p0.nedges;i++) 
		fp << edges[i] << "\n";

	return(OK);
}
/***************************************************************************/
int Graph::input(char *path) 
{
	int i,n,lineno=0,mode,showInfo;
	char s[128],msg[1024];
	ifstream fp(path,ios::in);

	showInfo=Sys.getShowInfo();

	// If we cannot open file
	if (!fp) {
		sprintf(msg,"Error:Cannot open file for reading:%s",path);
		put_error(msg);
		return(NO_CHANGE);
	}

	// Free all other data
	freeAll();

	// Print a message to the user so he does not get bored
	if (showInfo) {
		//sprintf(msg,"Loading graph:%s from file",path);
		//show_msg(msg);
	}

	fp >> s;
	if (strcmp(s,"Graph")!=0) {
		putError("Error:Wrong file type");
		return(-1);
	}

	fp >> s >> p0.npts;
	fp >> s >> p0.nedges;
	//fp >> s >> startNode;
	//fp >> s >> endNode;

	p0.directed=TRUE;				// Assume directional
	p0.negativeEdgeCosts=FALSE;		// Assume positive edge costs
	genSpace(p0);
	// Read lines forever until we have enough data	

	mode=0;		 
	for (i=0;i<p0.npts;i++)
		fp >> s >> nodes[i].x >> nodes[i].y;

	for (i=0;i<p0.nedges;i++) {
		fp >> edges[i];
		if (edges[i].cost<0.0) p0.negativeEdgeCosts=TRUE;
	}

	// Success
	Sys.addToRecentFiles(path);

	// Compute edgelists and reverse edgelists
	if (p0.npts && p0.nedges) {
		precomputation(p0);
		if (Sys.getShowGraphics()) createDrawing();
	}

	// Check if this graph is directed or not
	p0.directed=FALSE;
	if ((p0.nedges%2)==0) {
		for (i=0;i<p0.nedges;i++) {
			n=findEdge(edges[i].p2,edges[i].p1);
			if (n<0 || fabs(edges[i].cost-edges[n].cost)>0.00001) {
				p0.directed=TRUE;
				break;
			}
		}
	}
	else p0.directed=TRUE;
	
	return(OK);
}
/***************************************************************************/
// This computes reverse edges and edge lists
void Graph::precomputation(DataParameters param)
{
	int i;

	// Compute the reverse edges
	for (i=0;i<p0.nedges;i++) {
		reverseEdges[i].p1=edges[i].p2;
		reverseEdges[i].p2=edges[i].p1;
		reverseEdges[i].cost=edges[i].cost;
	}

	// This structure allows us to find all the edges from or to a node n
	computeEdgeLists();

	if (Sys.getShowGraphics()) 
		createDrawing();

	// ??? Debug
	if (Sys.getDebug()) checkForDuplicateEdge();

	//if (Sys.getShowInfo()) show_msg("Done with graph precomputation");
}
/****************************************************************************/
