/******************************************************************************/
/*
	GraphSP.cpp
	Created Oct, 2001
*/
/******************************************************************************/
#include "Graph.h"
#include "const.h"
#include "xmenus.h"
#include "ret.h"
/******************************************************************************/
// This solves the shortest path and fills in other information
int Graph::solveReverseShortestPath(void)
{
	int n;

	if (p0.npts==0 || labels==NULL) return(NO_CHANGE);

	// Solve the shortest path problem with Dijkstra or Bellman-Ford (Negative Costs)
	if (p0.negativeEdgeCosts==FALSE) n=solveReverseDijkstra(1);
	else {
		put_error("Still to do - solveReverseBellmanFord()");
		//n=solveReverseBellmanFord();
	}
	return(n);
}
/***************************************************************************/
// Solves shortest path using Dijkstra
int Graph::solveReverseDijkstra(int method)
{
	int i;
	int node,e,done,n,num,start;
	double c,cost;
	FibonacciHeap h;
	NodeLabels *p;					// Pointer to the node labels

	numDijkstras++;
	if (p0.npts==0) return(NOT_FOUND);

	// Init labels
	for (i=0;i<p0.npts;i++) {
		labels[i].fnode=NULL;				// Point to no heap node
		labels[i].reverseCost=INF;			// Reverse cost is infinite
		labels[i].reversePredEdge=(-1);		// No predecessor edge
		labels[i].scanned=FALSE;			// Not scanned
	}

	// Init endNode
	labels[endNode].reverseCost=0.0;
	labels[endNode].reversePredEdge=(-2);
	labels[endNode].scanned=FALSE;
	labels[endNode].fnode=h.insert_item(&(labels[endNode]),0.0); // Add first node to the heap
																											   
	// Now scanned all the nodes
	done=FALSE;
	while(FOREVER) {
		// Find the unscanned node with the lowest cost
		p=(NodeLabels *)(h.delete_min());
		if (p==NULL) break;
		node=p->num;
		/*
		if (node==(-1)) break;
		// Debug
		if (node<0 || node>p0.npts) {
			put_error("Graph::solveDijkstra - Invalid node found in heap");
			break;
		}
		*/

		// If this is the end node then there is a path to it 
		// so flag that we were successful
		if (node==startNode) {
			done=TRUE;
			if (method==0) break;
		}

		// Update all of its edges
		// Get the number of edges terminating at this node
		num=reverseEdgeList[node].n;
		// Get the starting point in the reverse index to the edges
		start=reverseEdgeList[node].start;
		for (i=0;i<num;i++) {
			e=reverseIndex[start+i];
			n=reverseEdges[e].p2;				// Connecting node

			// If we already scanned this node continue;
			if (labels[n].scanned) continue;

			// If this node is not used
			if (labels[n].valid==FALSE) continue;

			// If this edge cost is infinite - dont bother
			// The reverse edge is basically the same as the edge with the nodes switched
			// The edges are modified for subgraphs so use the cost associated with the edge
			cost=edges[e].cost;
			if (cost==INF) continue;

			// Update node n
			c=labels[node].reverseCost+cost;
			if (c<labels[n].reverseCost) {
				labels[n].reverseCost=c;
				labels[n].reversePredEdge=e;

				// If this node was not added to the heap, add it
				// Else just decrease the key value
				if (labels[n].fnode==NULL) labels[n].fnode=h.insert_item(&(labels[n]),c);
				else h.decrease_key((FNODE *)labels[n].fnode,c);
			}
		}

		// Flag that this node has be scanned
		labels[node].scanned=TRUE;
		numNodesScanned++;
	}

	if (done==FALSE) return(NOT_FOUND);	// Path not found 
	else return(OK);
}
/***************************************************************************/
// Solves shortest path using Dijkstra
// Note: We pass information since this same routine is used to solve the reverse path
// Method = 0 stop when destination node found
// Method = 1 solve for every node
int Graph::solveDijkstra(int method)
{
	int i,node,e,done,n,num,start;
	double c,cost;
	FibonacciHeap h;
	NodeLabels *p;						// List of node references to store in heap

	numDijkstras++;						// For statistics
	if (p0.npts==0) return(NOT_FOUND);

	// Init labels
	for (i=0;i<p0.npts;i++) {
		labels[i].fnode=NULL;
		labels[i].cost=INF;
		labels[i].predEdge=(-1);
		labels[i].scanned=FALSE;
		labels[i].path=(-1);
	}

	// Init startNode label
	labels[startNode].cost=0.0;
	labels[startNode].predEdge=(-2);
	labels[startNode].scanned=FALSE;
	labels[startNode].fnode=h.insert_item(&(labels[startNode]),0.0); // Add startNode to the heap
																											   
	// Now scanned all the nodes
	done=FALSE;
	while(FOREVER) {
		// Find the unscanned node with the lowest cost
		p=(NodeLabels *)(h.delete_min());
		if (p==NULL) break;
		node=p->num;
		if (node<0) break;

		/*
		// Debug
		if (node<0 || node>p0.npts) {
			put_error("Graph::solveDijkstra - Invalid node found in heap");
			break;
		}
		*/

		// If this is the end node then there is a path to it 
		// so flag that we were successful
		if (node==endNode) {
			done=TRUE;
			if (method==0) break;
		}

		// Update all of its edges
		num=forwardEdgeList[node].n;
		start=forwardEdgeList[node].start;
		for (i=0;i<num;i++) {
			e=forwardIndex[start+i];
			n=edges[e].p2;				// Connecting node

			// If we already scanned this node continue;
			if (labels[n].scanned) continue;

			// If this node is not used
			if (labels[n].valid==FALSE) continue;

			// If this edge cost is infinite - dont bother
			cost=edges[e].cost;
			if (cost==INF) continue;

			// Update node n
			c=labels[node].cost+cost;
			if (c<labels[n].cost) {
				labels[n].cost=c;
				labels[n].predEdge=e;

				// If this node was not added to the heap, add it
				// Else just decrease the key value
				if (labels[n].fnode==NULL) 
					labels[n].fnode=h.insert_item(&(labels[n]),c);
				else h.decrease_key((FNODE *)labels[n].fnode,c);
			}
		}

		// Flag that this node has be scanned
		labels[node].scanned=TRUE;
		numNodesScanned++;
	}

	if (done==FALSE) return(NOT_FOUND);	// Path not found 
	else return(OK);
}
/***************************************************************************/
int Graph::solveDijkstra(int method, NodeLabels *specificLabels)
{
	int i;
	int node,e,done,n,num,start;
	double c,cost;
	FibonacciHeap h;
	NodeLabels *p;					// Pointer to node labels

	numDijkstras++;
	if (p0.npts==0) return(NOT_FOUND);

	// Init labels
	for (i=0;i<p0.npts;i++) {
		specificLabels[i].fnode=NULL;
		specificLabels[i].cost=INF;
		specificLabels[i].predEdge=(-1);
		specificLabels[i].scanned=FALSE;
	}

	// Start node different
	specificLabels[startNode].cost=0.0;
	specificLabels[startNode].predEdge=(-2);
	specificLabels[startNode].scanned=FALSE;
	// Add startNode to the heap
	specificLabels[startNode].fnode=h.insert_item(&(specificLabels[startNode]),0.0); 
																											   
	// Now scanned all the nodes
	done=FALSE;
	while(FOREVER) {
		// Find the unscanned node with the lowest cost
		p=(NodeLabels *)(h.delete_min());
		if (p==NULL) break;
		node=p->num;
		/*
		if (node==(-1)) break;
		// Debug
		if (node<0 || node>p0.npts) {
			put_error("Graph::solveDijkstra - Invalid node found in heap");
			break;
		}
		*/

		// If this is the end node then there is a path to it 
		// so flag that we were successful
		if (node==endNode) {
			done=TRUE;
			if (method==0) break;
		}

		// Update all of its edges
		num=forwardEdgeList[node].n;
		start=forwardEdgeList[node].start;
		for (i=0;i<num;i++) {
			e=forwardIndex[start+i];
			n=edges[e].p2;				// Connecting node

			// If we already scanned this node continue;
			if (specificLabels[n].scanned) continue;

			// If this node is not used
			if (specificLabels[n].valid==FALSE) continue;

			// If this edge cost is infinite - dont bother
			cost=edges[e].cost;
			if (cost==INF) continue;

			// Update node n
			c=specificLabels[node].cost+cost;
			if (c<specificLabels[n].cost) {
				specificLabels[n].cost=c;
				specificLabels[n].predEdge=e;

				// If this node was not added to the heap, add it
				// Else just decrease the key value
				if (specificLabels[n].fnode==NULL) 
					specificLabels[n].fnode=h.insert_item(&(specificLabels[n]),c);
				else h.decrease_key((FNODE *)specificLabels[n].fnode,c);
			}
		}

		// Flag that this node has be scanned
		specificLabels[node].scanned=TRUE;
		numNodesScanned++;
	}

	if (done==FALSE) return(NOT_FOUND);	// Path not found 
	else return(OK);
}
/***************************************************************************/
// Uses the fastest correct algorithm to solve for the shortest path
int Graph::solveShortestPath(int method)
{
	int n;

	// Solve for the shortest path with the best algorithm
	if (p0.negativeEdgeCosts==FALSE) n=solveDijkstra(method);
	else n=solveBellmanFord();
									
	if (n==OK) 
		computePath(labels,endNode,shortestPath);	// Compute the path
	labelsValid=TRUE;
	return(n);
}
/***************************************************************************/
// Solves the shortest path using Dijkstra
int Graph::solveShortestPathDijkstra(int method)
{
	int n;
	n=solveDijkstra(method);						// Compute labels
	if (n==OK) 
		computePath(labels,endNode,shortestPath);	// Compute path
	labelsValid=TRUE;
	return(n);
}
/***************************************************************************/
// Solves the shortest path using Dijkstra
int Graph::solveShortestPathBellmanFord(void)
{
	int n;
	n=solveBellmanFord(startNode,endNode,p0.npts,edges,p0.nedges,labels);
	if (n==OK) 
		computePath(labels,endNode,shortestPath);	// Compute path
	labelsValid=TRUE;
	return(n);
}
/***************************************************************************/
// Solve BellmanFord for our graph only
int Graph::solveBellmanFord(void)
{
	int n;
	n=solveBellmanFord(startNode,endNode,p0.npts,edges,p0.nedges,labels);
	return(n);
}
/***************************************************************************/
// Solve a generic graph for the shortest path using BellmanFord
int Graph::solveBellmanFord(
	int startNode0, 
	int endNode0,
	int npts0, 
	Edge *edges0,
	int nedges0,
	NodeLabels *labels0)
{
	int i;
	int e,u,v,n;

	// Check that we some points
	if (npts0==0) return(NOT_FOUND);

	// Init step
	for (i=0;i<npts0;i++) {
		labels0[i].cost=INF;
		labels0[i].predEdge=(-1);
		labels0[i].scanned=FALSE;
	}

	// Init start node
	labels0[startNode0].cost=0;
	labels0[startNode0].predEdge=(-2);
	labels0[startNode0].scanned=FALSE;

	// Run the algorithm O(nm)
	for (i=0;i<npts0-1;i++) {
		for (e=0;e<nedges0;e++) {
			u=edges0[e].p1;
			v=edges0[e].p2;
			if (labels0[v].cost>(labels0[u].cost+edges0[e].cost)) {
				labels0[v].cost=labels0[u].cost+edges0[e].cost;
				labels0[v].predEdge=e;
			}
		}
	}

	// Check for a negative cycle
	n=checkForNegativeCycle(labels0,edges0,nedges0);
	if (n>=0) {
		return(NEGATIVE_CYCLE);
	}
	else if (labels0[endNode0].predEdge>=0) return(OK);
	else return(NOT_FOUND);
}
/***************************************************************************/
