/******************************************************************************/
/*
	Path.cpp
	Written by Matthew Maxel
	Created March 4, 2002
*/
/****************************************************************************/
// Path is a list of vertecies or edge and an associated cost
/****************************************************************************/
#include <iostream>
#include <string>
#include <sstream>
#include <assert.h>
#include "Path.h"
#include "ret.h"
#include "const.h"
/****************************************************************************/
using std::string;
/****************************************************************************/
Path::Path(void) 
{
	nList=0;
	list=NULL;						   
	cost=0.0;
}
/******************************************************************************/
Path::Path(int size) 
{
	nList=0;
	list=NULL;
	cost=0.0;
	if (size>0) setSize(size);
}
/******************************************************************************/
Path::Path(const Path& p)
{
	(*this)=p;
}
/******************************************************************************/
Path &Path::operator=(const Path& p)
{
	int i;

	cost=p.cost;
	list=NULL;
	if (p.nList) 
		setSize(p.nList);
	if (p.list)	
		for (i=0;i<p.nList;i++) list[i]=p.list[i];
	nList=p.nList;
	return(*this);
}
/******************************************************************************/
Path::~Path(void)
{
	// Delete the list 
	freeList();
}
/******************************************************************************/
Path Path::operator+(const Path& p)
{
	int i,n=nList+p.nList;
	Path path(n);
	for (i=0;i<nList;i++) path.list[i]=list[i];
	for (i=0;i<p.nList;i++) path.list[i+nList]=p.list[i];
	path.cost=cost+p.cost;
	path.nList=n;
	return path;
}
/******************************************************************************/
// Merges this path, node, and path2 into path pathOut
// Assumes this path and path2 are valid
int Path::merge(int list0, double cost0, Path *path2, Path *pathOut)
{
	int i;

	pathOut->setSize(nList+1+path2->nList);
	pathOut->nList=nList+1+path2->nList;

	for (i=0;i<nList;i++) pathOut->list[i]=list[i];
	pathOut->list[nList]=list0;
	for (i=0;i<path2->nList;i++) pathOut->list[nList+1+i]=path2->list[i];
	pathOut->cost=cost+cost0+path2->cost;
	return(OK);
}
/***************************************************************************/
int Path::operator==(Path *path)
{
	int i;
	if (nList!=path->nList) return(FALSE);
	for (i=0;i<path->nList;i++) {
		if (path->list[i]!=list[i]) return(FALSE);
	}
	return(TRUE);
}
/***************************************************************************/
int Path::operator!=(Path *path)
{
	if (this==path) return(FALSE);
	else return(TRUE);
}
/***************************************************************************/
int Path::operator==(const Path &path)
{
	int i;
	if (nList!=path.nList) return(FALSE);
	for (i=0;i<path.nList;i++) {
		if (path.list[i]!=list[i]) return(FALSE);
	}
	return(TRUE);
}
/***************************************************************************/
int Path::operator!=(const Path &path)
{
	if (this==(&path)) return(FALSE);
	else return(TRUE);
}
/***************************************************************************/
void Path::pushFront(int itemIn, double itemCost, Path &out)
{
	int i;

	out.setSize(nList+1);
	out.nList=nList+1;
	out.cost=cost+itemCost;
	out.list[0]=itemIn;
	for (i=0;i<nList;i++) 
		out.list[i+1]=list[i];
}
/***************************************************************************/
std::ostream &operator<<(std::ostream &output, const Path &p)
{
	int i;
	if (p.nList==0) output << "";
	else {
		//output << "Path " << "Cost:" << p.cost 
		//	<< " NumberOfNodes: " << p.nList << " ";
		for (i=0;i<p.nList-1;i++) 
			output << p.list[i] << " ";
		output << p.list[i] << "\0";
	}
	return(output);
}
/***************************************************************************/
void Path::setSize(int n)
{
	freeList();
	if (n) {
		list=new int[n];
		assert(list!=NULL);
	}
}
/***************************************************************************/
// Careful this sets nList to 0
void Path::freeList(void)
{
	if (list!=NULL) 
		delete list;
	list=NULL;
	nList=0;
}
/***************************************************************************/
