/***************************************************************************/
#include <stdio.h>
#include <string.h>
//#include <direct.h>
#include "system.h"
#include "const.h"
#include "ret.h"
#include "xmenus.h"
#include "resource.h"
/***************************************************************************/
System::System(void) 
{
	int i;
	graphFile[0]=0;		// Name of the current file
	//_getcwd(programPath,255); // Path of program i.e. C:\algo
	for (i=0;i<4;i++) 
		recentFiles[i][0]=0;	// List of recent files
	lastFile[0]=0;			// Last file accessed
	printing=0;				// True if printing
	printSize=1;			// Relative pixel sizes of the printer vs the screen 
	backgroundColor=0;		// Background color of the screen
	debug=FALSE;			// Turn debugging on
	level=0;				// Current level of recursion
	recall=(-1);			// Menu item to keep calling
	showGraphics=TRUE;		// Show graphics
	changed=FALSE;			// Something changed
	logging=FALSE;
	logFP=NULL;
	labelPoints=TRUE;
	labelEdges=TRUE;
	showNodeLabels=FALSE;
	showInfo=TRUE;			// Info messages
}
/******************************************************************************/
System::~System(void)
{
	// Delete other information/links
}
/******************************************************************************/
void System::getGraphFile(char *name)
{
	strcpy(name,graphFile);
}
/******************************************************************************/
void System::setGraphFile(char *name)
{
	strcpy(graphFile,name);
}
/******************************************************************************/
void System::getProgramPath(char *name)
{
	strcpy(name,programPath);
}
/******************************************************************************/
void System::getPath(char *path, char *fullpath)
{
	sprintf(fullpath,"%s%s",programPath,path);
}
/******************************************************************************/
void System::setProgramPath(char *name)
{
	strcpy(programPath,name);
}
/******************************************************************************/
void System::getRecentFiles(int n, char *name)
{
	strcpy(name,recentFiles[n]);
}
/******************************************************************************/
void System::setRecentFiles(int n, char *name)
{
	strcpy(recentFiles[n],name);
}
/******************************************************************************/
void System::getLastFile(char *name)
{
	strcpy(name,lastFile);
}
/******************************************************************************/
void System::setLastFile(char *name)
{
	strcpy(lastFile,name);
}
/******************************************************************************/
int System::getPrinting(void)
{
	return(printing);
}
/******************************************************************************/
void System::setPrinting(int n)
{
	printing=n;
}
/******************************************************************************/
int System::getPrintSize(void)
{
	return(printSize);
}
/******************************************************************************/
void System::setPrintSize(int size)
{
	printSize=size;
}
/******************************************************************************/
void System::setBackgroundColor(long color)
{
	backgroundColor=color;
}
/******************************************************************************/
long System::getBackgroundColor(void)
{
	return(backgroundColor);
}
/******************************************************************************/
void System::setDebug(int n)
{
	debug=n;
}
/******************************************************************************/
int System::getDebug(void)
{
	return(debug);
}
/******************************************************************************/
void System::setLevel(int n)
{
	level=n;
}
/******************************************************************************/
int System::getLevel(void)
{
	return(level);
}
/******************************************************************************/
void System::setShowGraphics(int n)
{
	showGraphics=n;
}
/******************************************************************************/
int System::getShowGraphics(void)
{
	return(showGraphics);
}
/******************************************************************************/
void System::setLabelPoints(int n)
{
	labelPoints=n;
}
/******************************************************************************/
int System::getLabelPoints(void)
{
	return(labelPoints);
}
/******************************************************************************/
void System::setLabelEdges(int n)
{
	labelEdges=n;
}
/******************************************************************************/
int System::getLabelEdges(void)
{
	return(labelEdges);
}
/******************************************************************************/
void System::setShowNodeLabels(int n)
{
	showNodeLabels=n;
}
/******************************************************************************/
int System::getShowNodeLabels(void)
{
	return(showNodeLabels);
}
/******************************************************************************/
int System::getRecall(void)
{
	return(recall);
}
/******************************************************************************/
void System::setRecall(int n)
{
	recall=n;
}
/******************************************************************************/
int System::addToRecentFiles(char *name)
{
	int i,j;
	int found=FALSE;
	char file[256];

	for (i=0;i<4;i++) {
		getRecentFiles(i,file);
	    if (strcasecmp(name,file)==0) {
			found=TRUE;

			for (j=i-1;j>=0;j--) {
				getRecentFiles(j,file);
				setRecentFiles(j+1,file);
			}

			setRecentFiles(0,name);
			break;
		}
	}
	if (!found) {
		for (i=2;i>=0;i--) {
			getRecentFiles(i,file);
			setRecentFiles(i+1,file);
		}
		setRecentFiles(0,name);
	}
	setMenuRecentFiles();
	return(OK);
}
/***************************************************************************/
void System::setMenuRecentFiles(void)
{
	int i;
	char msg[256],file[256];

	for (i=0;i<4;i++) {
		getRecentFiles(i,file);
		sprintf(msg,"&%1d %s",i+1,file);
		set_menu(ID_FILE_1+i,msg,0);
	}
}
/***************************************************************************/
void System::setChanged(int changed_in)
{
	changed=changed_in;
}
/***************************************************************************/
int System::getChanged(void)
{
	return(changed);
}
/***************************************************************************/
int System::getLogging(void)
{
	return(logging);
}
/***************************************************************************/
void System::startLogging(void)
{
	if (logFP!=NULL) return;
	logFP=fopen("log.txt","w");
	if (logFP==NULL) 
		put_error("Error:Cannot open log.txt file for writing");
	else 
		logging=TRUE;
}
/***************************************************************************/
void System::stopLogging(void)
{
	logging=FALSE;
	if (logFP==NULL) return;
	fclose(logFP);
	logFP=NULL;
}
/***************************************************************************/
void System::logString(char *s)
{
	if (logFP) fprintf(logFP,s);
}
/***************************************************************************/
void System::logNL(void)
{
	if (logFP) fprintf(logFP,"\n");
}
/***************************************************************************/
int System::getShowInfo(void)
{
	return(showInfo);
}
/***************************************************************************/
void System::setShowInfo(int showInfoIn)
{
	showInfo=showInfoIn;
}
/***************************************************************************/
