/******************************************************************************/
/*
	graphYen.cpp
*/
/******************************************************************************/
#include <math.h>
#include "Graph.h"
#include "const.h"
#include "ret.h"
#include "xmenus.h"
#include "xrout.h"
/******************************************************************************/
int Graph::computeYenKSP(int K, Path *pathsOut)
{
	int k=0,n;
	FibonacciHeap h;

	numDijkstras=0;
	numNodesScanned=0;

	// Compute A1
	n=solveShortestPath(0);
	if (n!=OK) return(0);// No paths

	// Add this path to the heap
	addYenPathToHeap(&h,shortestPath);

	// Loop while we are searching for paths
	while(FOREVER) {
		n=computeYenNextPath(&h,k,pathsOut);
		if (n!=OK || k>=K) break;
	}

	// Return the actual number of paths generated
	return(k);
}
/***************************************************************************/
int Graph::computeYenNextPath(FibonacciHeap *h, int &k, Path *pathsOut)
{
	int i,j,m,e,n;
	int lastLen,last,node,sameSubPath;
	Graph H;
	Edge edge;
	Path *p,newPath,spur;

	// Keep popping until we get a unique path or we run out of paths
	while(FOREVER) {
		// Pop the shortest path from the heap
		p=(Path *)(h->delete_min());
		// If no more paths in heap return since we cannot generate any more
		if (p==NULL) return(NOT_FOUND);

		if (k==0 || fabs(p->cost-pathsOut[k-1].cost)>0.0001) break;
	}

	// Check if we inserted a bad path into the heap
	if (p->nList==0) { 	
		put_error("Error:Bad path found in heap");
		//Sys.stopLogging();
		return(NOT_FOUND);
	}

	// Steal data/memory from p
	pathsOut[k].list=p->list;
	pathsOut[k].cost=p->cost;
	pathsOut[k].nList=p->nList;
	p->list=NULL;
	k++;
	delete p;	// Dont need this path anymore

	// Now find new possiblities for paths
	last=k-1;
	lastLen=pathsOut[last].nList;

	// Copy G to H
	copy(&H);

	// For all viable points find a new shortest path
	for (i=0;i<lastLen;i++) {

		// Find if subpath 1-i is the same as some other path
		for (m=0;m<=last;m++) {

			// Assume same SubPath
			sameSubPath=TRUE;
			for (j=0;j<i;j++) {
				if (pathsOut[m].list[j]!=pathsOut[last].list[j]) {
					sameSubPath=FALSE;
					break;
				}			
			}

			// If this path follows the same subpath block its deviation edge
			if (sameSubPath==TRUE && pathsOut[m].nList>i)
				H.setEdgeInvalid(pathsOut[m].list[i]);
		}

		// Find the new start node
		if (i==lastLen) {
			e=pathsOut[last].list[lastLen-1];
			node=edges[e].p2;
		}
		else {
			e=pathsOut[last].list[i];
			node=edges[e].p1;
		}

		H.setStartNode(node);
		n=H.solveShortestPath(0);
		if (n==OK) {
			H.getShortestPath(spur);
			newPath.setSize(i+spur.nList);
			newPath.cost=0.0;
			newPath.nList=i+spur.nList;

			for (j=0;j<i;j++) {
				e=pathsOut[last].list[j];
				newPath.list[j]=e;
				newPath.cost+=edges[e].cost;
			}
			for (j=0;j<spur.nList;j++) {
				e=spur.list[j];
				newPath.list[j+i]=e;
				newPath.cost+=edges[e].cost;
			}

			addYenPathToHeap(h,newPath);
		}

		// Cannot use this node anymore	(or we would loop);
		H.setNodeInvalid(node);
	}

	numDijkstras+=H.getNumDijkstras();
	numNodesScanned+=H.getNumNodesScanned();
	H.release();	// Release computed information that belongs to G

	return(OK);
}
/***************************************************************************/
void Graph::addYenPathToHeap(FibonacciHeap *h, const Path& pathIn)
{
	Path *path=new Path(pathIn);
	h->insert_item(path,path->cost);
}
/***************************************************************************/
