/***************************************************************************/
#ifndef System_h
#define System_h
/***************************************************************************/
struct ProgramModes {
	unsigned int drawing:1;			 // True if drawing
	unsigned int printing:1;         // True if printing
};
/***************************************************************************/
// This class holds all of the system variables
class System {
private:
	char graphFile[256];		// Name of the current file
	char programPath[256];		// Path of program i.e. C:\algo
	char recentFiles[4][256];	// List of recent files
	char lastFile[256];			// Last file accessed
	int printing;				// True if printing
	int printSize;				// Relative pixel sizes of the printer vs the screen 
	long backgroundColor;		// Background color of the screen
	int debug;					// Turn debugging on
	int level;					// Current level of recursion
	int recall;					// Menu item to keep calling
	int showGraphics;			// Show graphics
	int changed;
	int logging;				// Logging data
	FILE *logFP;				// Logging file
	int labelEdges;
	int labelPoints;
	int showNodeLabels;
	int showInfo;				// Show Informational Message
public:
	System(void);
	~System(void);
	void getGraphFile(char *name);
	void setGraphFile(char *name);
	void getProgramPath(char *name);
	void getPath(char *path, char *fullpath);
	void setProgramPath(char *name);
	void getRecentFiles(int n, char *name);
	void setRecentFiles(int n, char *name);
	void getLastFile(char *name);
	void setLastFile(char *name);
	void setPrinting(int n);
	int getPrinting(void);
	int getPrgMode(void);
	void setPrgMode(int mode);
	int getPrintSize(void);
	void setPrintSize(int size);
	void setBackgroundColor(long color);
	long getBackgroundColor(void);
	void setDebug(int n);
	int getDebug(void);
	void setLevel(int n);
	int getLevel(void);
	void setShowGraphics(int n);
	int getShowGraphics(void);
	int getRecall(void);
	void setRecall(int n);
	int addToRecentFiles(char *name);
	void setMenuRecentFiles(void);
	void setChanged(int changed);
	int getChanged(void);
	int getLogging(void);
	void startLogging(void);
	void stopLogging(void);
	void logString(char *s);
	void setLabelPoints(int n);
	int getLabelPoints(void);
	void setLabelEdges(int n);
	int getLabelEdges(void);
	void setShowNodeLabels(int n);
	int getShowNodeLabels(void);
	void logNL(void);
	int getShowInfo(void);
	void setShowInfo(int showInfoIn);
};
/***************************************************************************/
#endif
/***************************************************************************/
